/*************************************************************
 * ct_main.c
 *
 * CT timer main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2010. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_CT
#include <ct_api.h>


#define	CT_MAIN_LOOP_CT_MAX		0x0a
#define CT_GET_COUNT_VAL		0x0a

static volatile unsigned char	ctLoopCt;	//loop counter

void ctMain(void);
static void startCT(void);
static void stopCT(void);
void ctIntHandler(void)__attribute__ ((interrupt_handler));
void ctInt(void);


/*
 * CT clock timer sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void ctMain(void)
{
	printf("<<< Clock timer demonstration start >>>\n");

	// start CT
	startCT();

	ctLoopCt = 1U;
	while(ctLoopCt <= CT_MAIN_LOOP_CT_MAX) {
		asm( "halt" );
		asm( "nop" );
		ctLoopCt++;
	}

	// stop CT
	stopCT();

	printf("<<< Clock timer demonstration finish >>>\n");
}

/*
 * start CT timer for 1 sec interval.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startCT(void)
{
	//set CT initial
	CT_resetTimer();

	CT_initInt(CT_INT_LEVEL3);
	CT_controlInt(CT_32INT_DISABLE | CT_8INT_DISABLE | CT_2INT_ENABLE | CT_1INT_DISABLE);
	CT_setTimerRun(CT_TIMER_RUN);
}


/*
 * stop CT timer.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopCT(void)
{
	CT_setTimerRun(CT_TIMER_STOP);
	CT_controlInt(CT_32INT_DISABLE | CT_8INT_DISABLE | CT_2INT_DISABLE | CT_1INT_DISABLE);
	CT_initInt(CT_INT_LEVEL0);
}


/*
 * CT interrupt handler
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void ctIntHandler(void)
{
	// call interrupt process function
	ctInt();
}

/*
 * CT interrupt process
 * param	void
 * return	void
 * history	:	2010/04/05
 */
void ctInt(void)
{
	unsigned char	flgIntr;		// interrupt flag

	// get interrupt status
	CT_checkIntFlag(&flgIntr);

	if((flgIntr & CT_2INT_OCCURRED) == CT_2INT_OCCURRED) {
		printf("*** %01d.%01d sec ***\n", ctLoopCt / 2U, (ctLoopCt * 5U) % 10U);

		// reset ineterrupt flag
		CT_resetIntFlag(CT_2INT_RESET);
	}
}

#endif	// end of PE_CT
