/*************************************************************
 * swt_main.c
 *
 * stopwatch timer main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_SWT
#include <swt_api.h>


#define SWT_RUN_TIME_MIN	1		// loop count min
#define SWT_RUN_TIME_MAX	9		// loop count max


void swtMain(void);
static void startSWT1(void);
static void stopSWT(void);
void swtIntHandler(void)__attribute__ ((interrupt_handler));

static volatile unsigned char	loopCountMax;
static volatile unsigned char	swtloopCt;


/*
 * SWT stop watch sample program.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void swtMain(void)
{
	unsigned char	inputString[3];							// user input data buffer

	printf("<<< Stop watch timer demonstration start >>>\n");
	printf("Please input time 1-9[sec]\n");

	//get user input
	do {
		fgets((char*)&inputString[0], sizeof(inputString), stdin);
		sscanf(inputString, "%d", (char*)&loopCountMax);

	} while( (loopCountMax < SWT_RUN_TIME_MIN) || (loopCountMax > SWT_RUN_TIME_MAX) );

	printf("Start stopwatch timer...\n\n");

	// start SWT
	startSWT1();

	swtloopCt = 1;
	while(swtloopCt < loopCountMax) {
		//wait loop until meet the conditions.
		;
	}

	printf("%d sec passed\n", swtloopCt);

	// stop SWT
	stopSWT();

	printf("<<< Stop watch timer demonstration finish >>>\n");
}

/*
 * start SWT timer for 1 sec interval.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startSWT1(void)
{
	SWT_resetTimer();
	
	SWT_initInt(SWT_INT_LEVEL3);
	SWT_controlInt(SWT_1INT_ENABLE | SWT_10INT_DISABLE | SWT_100INT_DISABLE);
	SWT_setTimerRun(SWT_TIMER_RUN);
}


/*
 * Stop all SWT timer.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopSWT(void)
{
	SWT_setTimerRun(SWT_TIMER_STOP);
	SWT_controlInt(SWT_1INT_DISABLE | SWT_10INT_DISABLE | SWT_100INT_DISABLE);
	SWT_initInt(SWT_INT_LEVEL0);
}

/*
 * SWT interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void swtIntHandler(void)
{
	unsigned char	intFlag;		// interrupt flag

	// get interrupt flag
	SWT_checkIntFlag(&intFlag);

	if(swtloopCt <= loopCountMax) {
		if((intFlag & SWT_1INT_OCCURRED) == SWT_1INT_OCCURRED) {
			swtloopCt++;
		}
	}

	// reset interrupt flag
	SWT_resetIntFlag(intFlag);
}
#endif	// end of PE_SWT
