/*************************************************************
 * t16e_main.c
 *
 * PWM timer main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_T16E
#include <mux_api.h>
#include <t16e_api.h>

#define	T16E_MAIN_LOOP_CT_MAX	5					// loop counter max
#define	T16E_MAIN_A				0x00					// compare A index
#define	T16E_MAIN_B				T16E_MAIN_A + 1		// compare B index
#define	T16E_MAIN_CMP_MATCH_NO	T16E_MAIN_B + 1		// a number of compare match registers
#define	T16E_MAIN_CH			T16E_CH0			// T16 ch

static unsigned short	t16eBufCounter[T16E_MAIN_CMP_MATCH_NO][T16E_MAIN_LOOP_CT_MAX];	// counter record buffer
static volatile unsigned char	t16eLoopCt;									// loop counter

// prototype
void t16eMain(void);
static void startT16E(unsigned char chNo);
static void stopT16E(unsigned char chNo);
void t16eIntHandler(void)__attribute__((interrupt_handler));

/*
 * T16E timer sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void t16eMain(void)
{
	printf("<<< PWM timer demonstration start >>>\n");
	
	// start T16E timer
	startT16E(T16E_MAIN_CH);

	t16eLoopCt = 0;
	while(t16eLoopCt < T16E_MAIN_LOOP_CT_MAX) {
		asm( "halt" );
		asm( "nop" );
		t16eLoopCt++;
	}

	// stop T16E timer
	stopT16E(T16E_MAIN_CH);

	t16eLoopCt = 0;
	while(t16eLoopCt < T16E_MAIN_LOOP_CT_MAX) {
		t16eLoopCt++;
		printf("***PWM compare A interrupt : %4d ***\n",
				t16eBufCounter[T16E_MAIN_A][t16eLoopCt - 1U]);
		printf("***PWM compare B interrupt : %4d ***\n",
				t16eBufCounter[T16E_MAIN_B][t16eLoopCt - 1U]);
	}

	// clear port
	MUX_setT16Eport(T16E_CH0, MUX_OFF);

	printf("<<< PWM timer demonstration finish >>>\n");
}

/*
 * start T16E timer for 1 sec interval.
 * param	T16 channel number.
 * return	void
 * history	:	2009/12/25
 */
static void startT16E(unsigned char chNo)
{
	T16E_TIM_MODE	mode;	// timer mode setting
	T16E_INIT_PARAM	param;	// timer parameter

	mode.initol	= T16E_TIMMODE_TOUT3L;
	mode.selfm	= T16E_TIMMODE_NORMAL;
	mode.cbufen	= T16E_TIMMODE_ENABLE;
	mode.invout	= T16E_TIMMODE_NORMAL;
	mode.clksel	= T16E_TIMMODE_INTERNAL;
	mode.outen	= T16E_TIMMODE_ENABLE;


	param.dataA	= OSC3_CLK/16384/4;
	param.dataB	= OSC3_CLK/16384/2;
	param.selectClk	= T16E_CNTCLK_16384;
	param.data	= 0;

	T16E_init(chNo, &mode, &param);
	T16E_initInt(chNo, T16E_INT_LEVEL3);
	T16E_controlInt(chNo, T16E_CMPAINT_ENABLE | T16E_CMPBINT_ENABLE);
	T16E_setTimerRun(chNo, T16E_TIMER_RUN);
}

/*
 * stop T16E timer.
 * param	T16 channel number.
 * return	void
 * history	:	2009/12/25
 */
static void stopT16E(unsigned char chNo)
{
	T16E_setTimerRun(chNo, T16E_TIMER_STOP);
	T16E_controlInt(chNo, T16E_CMPAINT_DISABLE | T16E_CMPBINT_DISABLE);
	T16E_initInt(chNo, T16E_INT_LEVEL0);
}

/*
 * T16E interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void t16eIntHandler(void)
{
	unsigned short	intFlag;		// interrupt flag

	// get interrupt status
	T16E_checkIntFlag(T16E_MAIN_CH, &intFlag);
	
	// compare A interrupt
	if((intFlag & T16E_AINT_OCCURRED) == T16E_AINT_OCCURRED) {
		T16E_getCounterData(T16E_MAIN_CH, &t16eBufCounter[T16E_MAIN_A][t16eLoopCt]);
	}

	// compare B interrupt
	if((intFlag & T16E_BINT_OCCURRED) == T16E_BINT_OCCURRED) {
		T16E_getCounterData(T16E_MAIN_CH, &t16eBufCounter[T16E_MAIN_B][t16eLoopCt]);
	}
	
	// reset interrupt flag
	T16E_resetIntFlag(T16E_MAIN_CH, intFlag);	
	
}

#endif	// end of PE_T16E
