/*************************************************************
 * uart_iosc_main.c
 *
 * UART IOSC main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_UART_IOSC
#include <string.h>

#include <mux_api.h>
#include <t8f_api.h>
#include <t8osc1_api.h>
#include <t16_api.h>
#include <osc_api.h>
#include <uart_api.h>

#include <uart_main.h>

#define	UART_IOSC_T8OSC1_CLKDIV	1			// T8OSC1 CLOCK DIVISOR
#define	UART_IOSC_T8OSC1_CMP	7			//
#define	UART_IOSC_BAUDRATE	115200			// baudrate

/* send data */
#define	UART_IOSC_INITDATA	0x21U
#define	UART_IOSC_ENDDATA	0x7EU

#define UART_IOSC_MAIN_CH			UART_CH0				// UART ch

// prototype
void uartIOSCMain(void);
static void startIOSC(void);
static void setT8O(unsigned char compare);
static void stopT8O(void);
static void setT16(unsigned char chNo);
static void calibUartIOSC(unsigned char *tr, unsigned char *tfmd, unsigned long baudrate);
void t8oCallBackFunction(void);

extern void startUart(unsigned char chNo);
extern void stopUart(unsigned char chNo);
extern void startUartT8F(unsigned char chNo, unsigned char tr, unsigned char tfmd);
extern void sendUARTConnectionData(unsigned char chNo);
extern unsigned char sendUARTData(unsigned char chNo, unsigned char *buf, unsigned char len);
extern void startAllOSC(void);

// T8OSC1 timer functions
extern void t8oSetCallBackFunc(void(*name)(void));
extern void t8oClearCallBackFunc(void);

extern volatile unsigned char	flgConnectionStatus;	// connection status
extern volatile UART_MAIN_PARAM paramUart;
extern volatile unsigned short	sendWait;

static unsigned char strReceiveData[UART_IOSC_ENDDATA - UART_IOSC_INITDATA + 2];	//receve data buffer

/*
 * UART sample program
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void uartIOSCMain(void)
{
	unsigned char strSendData[UART_IOSC_ENDDATA - UART_IOSC_INITDATA + 2];		// send data buffer
	unsigned char tr;		// T8F_TR register
	unsigned char tfmd;		// T8F_TFMD register

	unsigned char	i;													// counter
	unsigned char	flag;												// check flag

	printf("<<< UART IOSC demonstration start >>>\n");

	// initialize send data
	for(i = UART_IOSC_INITDATA; i < UART_IOSC_ENDDATA; i++) {
		strSendData[i - UART_IOSC_INITDATA] = i;
	}
	strSendData[i - UART_IOSC_INITDATA] = '\n';

	memset(strReceiveData, '\0', sizeof(strReceiveData));

	startIOSC();														// enable IOSC

	// get IOSC prescaler parameter set to T8F
	calibUartIOSC(&tr, &tfmd, UART_IOSC_BAUDRATE);

	// start T8F
	startUartT8F(UART_IOSC_MAIN_CH, tr, tfmd);

	// start UART
	paramUart.counterRx = 0;
	paramUart.counterMaxRx = (UART_IOSC_ENDDATA - UART_IOSC_INITDATA + 1);
	paramUart.dataRx = &strReceiveData[0];

	flgConnectionStatus = UART_NOT_CONNECTED;
	sendWait = 0;
	startUart(UART_IOSC_MAIN_CH);

	// wait connection.
	printf("waiting connection.\n");
	sendUARTConnectionData(UART_IOSC_MAIN_CH);

	// data send
	sendUARTData(UART_IOSC_MAIN_CH, &strSendData[0], (UART_IOSC_ENDDATA - UART_IOSC_INITDATA + 1));

	// wait all data receive.
	while(paramUart.counterRx < paramUart.counterMaxRx) {
		;
	}
	while(1) {
		UART_checkReceiveFlag(UART_IOSC_MAIN_CH, &flag);
		if((flag & UART_RXDATA_READY) == UART_RXDATA_EMPTY) {
			break;
		}
	}
	printf("connected.\n");
	printf("*** send data ***\n");

	// stop UART
	stopUart(UART_IOSC_MAIN_CH);

	// display data
	strReceiveData[paramUart.counterMaxRx] = '\0';
	printf("*** received data ***\n");
	printf("%s",strReceiveData);

	// stop T8F
	T8F_setTimerRun(T8F_CH0, T8F_TIMER_STOP);

	// start all OSC
	startAllOSC();

	// clear port
	MUX_setUARTport(UART_IOSC_MAIN_CH, MUX_UART_CLK_INTERNAL, MUX_OFF);

	printf("<<< UART IOSC demonstration finish >>>\n");
}

/*
 * start IOSC.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void startIOSC(void)
{
	CLG_OSC_setWaitCycle(CLG_OSC_SELIOSC_64, CLG_OSC_SELOSC3_512);			// set wait cycle
	CLG_OSC_controlOscillation(CLG_OSC_IOSC_ENABLE | CLG_OSC_OSC1_ENABLE | CLG_OSC_OSC3_ENABLE);	// start all OSC

	CLG_OSC_setClockSource(CLG_OSC_SELHSCLK_IOSC, CLG_OSC_SELCLKSRC_HSCLK);	// change clock IOSC
}

/*
 * start T8OSC1 timer for UART.
 * param	unsigned char							compare data
 * return	void
 * history	:	2009/12/25
 */
static void setT8O(unsigned char compare)
{
	T8O_init(T8F_TIMMODE_REPEAT, CLG_OSC_T8OSC1CLK_1, compare);

	T8O_initInt(T8O_INT_LEVEL3);
	T8O_controlInt(T8O_INT_ENABLE);
}

/*
 * stop T8OSC1 timer for UART.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
static void stopT8O(void)
{
	T8O_controlInt(T8O_INT_DISABLE);
	T8O_initInt(T8O_INT_LEVEL0);
}

/*
 * start T16 timer for 1 sec interval.
 * param	T16 channel number.
 * return	void
 * history	:	2009/12/25
 */
static void setT16(unsigned char chNo)
{
	T16_TIM_MODE	mode;	// timer mode setting

	mode.ckactv	= T16_TIMMODE_ACTVL;
	mode.cksl	= T16_TIMMODE_INTERNAL;
	mode.trmd	= T16_TIMMODE_ONESHOT;
	T16_init(chNo, T16_CNTCLK_1, &mode, 0xFFFF);
}

/*
 * get T8F parameter using IOSC.
 * this function must be called while IOSC is provided to T16
 * and OSC1 is valid.
 * param	*tr		T8F reload data register value.
 * param	*tfmd	T8F fine mode setup bit value.
 * return	void
 * history	:	2009/12/25
 */
static void calibUartIOSC(unsigned char *tr, unsigned char *tfmd, unsigned long baudrate)
{
	unsigned short	valT16Count;		// T16 timer counter data
	unsigned long	t16cnt;				// calculation work
	unsigned long	temp;				// calculation work

	// set a call back function to T8OSC1 interrupt handler
	t8oSetCallBackFunc(&t8oCallBackFunction);

	// set T8OSC1 and T16
	setT8O(UART_IOSC_T8OSC1_CMP);
	CLG_OSC_controlT8OSC1(CLG_OSC_T8OSC1_ENABLE);
	setT16(T16_CH0);

	// start T8OSC1 and T16
	T8O_setTimerRun(T8O_TIMER_RUN);

	// halt CPU and wait a OSC1 interrupt
	// T16 start
	asm( "halt" );
	asm( "nop" );
	// T16 stop
	asm( "halt" );
	asm( "nop" );

	// stop T8OSC1
	CLG_OSC_controlT8OSC1(CLG_OSC_T8OSC1_DISABLE);
	stopT8O();

	// unregister a call back function from T16 timer handler
	t8oClearCallBackFunc();

	// get T16 counter data
	T16_getCounterData(T16_CH0, &valT16Count);

	// calculate
	t16cnt = ~valT16Count;
	temp  = (t16cnt << 12) * 10 / baudrate + 5;
	temp /= 10;

	*tr = (unsigned char)(temp / 16U - 1U);
	*tfmd = (unsigned char)(temp % 16);
}

/*
 * T8o timer call back function.
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void t8oCallBackFunction(void)
{
	asm( "xld.b	%r0, [0x4226]" );
	asm( "xor		%r0, 0x01" );
	asm( "xld.b	[0x4226], %r0" );

	asm( "ld		%r0, 0x01" );
	asm( "xld.b	[0x50c4], %r0" );
}


#endif	// end of PE_UART_IOSC
