/*************************************************************
 * uart_osc3_main.c
 *
 * UART OSC3 main module
 *
 * Copyright (C)SEIKO EPSON CORPORATION 2009. All rights reserved.
 ************************************************************/

#include <stdio.h>
#include <s1c176xx.h>

#ifdef	PE_UART_OSC3
#include <string.h>

#include <mux_api.h>
#include <t8f_api.h>
#include <uart_api.h>

#include <uart_main.h>

/* send data */
#define	UART_OSC3_INITDATA	0x21U
#define	UART_OSC3_ENDDATA	0x7EU
#define	UART_OSC3_SENDWAIT	0x0002

#define UART_OSC3_MAIN_CH			UART_CH0				// UART ch

volatile unsigned char	flgConnectionStatus;	// connection status
volatile UART_MAIN_PARAM paramUart;
static 	unsigned char strReceiveData[UART_OSC3_ENDDATA - UART_OSC3_INITDATA + 2];	//receve data buffer
volatile unsigned short	sendWait;

/* prototype */
void uartOSC3Main(void);
void startUart(unsigned char chNo);
void stopUart(unsigned char chNo);
void startUartT8F(unsigned char chNo, unsigned char tr, unsigned char tfmd);
void sendUARTConnectionData(unsigned char chNo);
unsigned char sendUARTData(unsigned char chNo, unsigned char *buf, unsigned char len);
void uartIntHandler(void)__attribute__((interrupt_handler));


/*
 * UART OSC3 sample program
 * param	void
 * return	void
 * history	:	2010/06/15
 */
void uartOSC3Main(void)
{
	unsigned char strSendData[UART_OSC3_ENDDATA - UART_OSC3_INITDATA + 2];	// send data buffer
	unsigned char	i = 0;												// counter
	unsigned char	flag;												// check flag

	printf("<<< UART OSC3 demonstration start >>>\n");

	// initialize send data
	for(i = UART_OSC3_INITDATA; i < UART_OSC3_ENDDATA; i++) {
		strSendData[i - UART_OSC3_INITDATA] = i;
	}
	strSendData[i - UART_OSC3_INITDATA] = '\n';

	memset(strReceiveData, '\0', sizeof(strReceiveData));

	// start T8F
	startUartT8F(T8F_CH0, ((OSC3_CLK) / (115200 * 16) - 1), T8F_TIMMODE_PATTERN5);

	// start UART
	paramUart.counterRx = 0;
	paramUart.counterMaxRx = (UART_OSC3_ENDDATA - UART_OSC3_INITDATA + 1);
	paramUart.dataRx = &strReceiveData[0];

	flgConnectionStatus = UART_NOT_CONNECTED;
	sendWait = 0;
	startUart(UART_OSC3_MAIN_CH);

	// wait connection.
	printf("waiting connection.\n");
	sendUARTConnectionData(UART_OSC3_MAIN_CH);

	// data send
	sendUARTData(UART_OSC3_MAIN_CH, &strSendData[0], (UART_OSC3_ENDDATA - UART_OSC3_INITDATA + 1));

	// wait all data receive.
	while(paramUart.counterRx < paramUart.counterMaxRx) {
		;
	}
	while(1) {
		UART_checkReceiveFlag(UART_OSC3_MAIN_CH, &flag);
		if((flag & UART_RXDATA_READY) == UART_RXDATA_EMPTY) {
			break;
		}
	}
	printf("connected.\n");
	printf("*** send data ***\n");

	// stop UART
	stopUart(UART_OSC3_MAIN_CH);

	// display data
	strReceiveData[paramUart.counterMaxRx] = '\0';
	printf("*** received data ***\n");
	printf("%s", strReceiveData);

	// stop T8F
	T8F_setTimerRun(T8F_CH0, T8F_TIMER_STOP);

	// clear port
	MUX_setUARTport(UART_OSC3_MAIN_CH, MUX_UART_CLK_INTERNAL, MUX_OFF);

	printf("<<< UART OSC3 demonstration finish >>>\n");
}

/*
 * start UART.
 * param	UART channel number.
 * return	void
 * history	:	2009/12/25
 */
void startUart(unsigned char chNo)
{
	UART_INIT_PARAM		param;											// initialize parameter

	param.dataLen = UART_DATALEN_8BIT;
	param.parity = UART_PARITY_NONE;
	param.stopBit = UART_STOPBIT1;
	param.inputClk = UART_CLK_INTERNAL;
	UART_init(chNo, &param);

	UART_initInt(chNo, UART_INT_LEVEL3);
	UART_controlInt(chNo, (UART_RXERRORINT_DISABLE | UART_RXINT_ENABLE | UART_TXINT_DISABLE), UART_RXINT_1BYTE);
}

/*
 * stop UART.
 * param	UART channel number.
 * return	void
 * history	:	2009/12/25
 */
void stopUart(unsigned char chNo)
{
	UART_setComEnable(chNo, UART_CTL_DISABLE);
	UART_controlInt(chNo, (UART_RXERRORINT_DISABLE | UART_RXINT_DISABLE | UART_TXINT_DISABLE), UART_RXINT_1BYTE);
	UART_initInt(chNo, UART_INT_LEVEL0);
}

/*
 * start T8F timer for UART.
 * param	T8F channel number.
 * return	void
 * history	:	2009/12/25
 */
void startUartT8F(unsigned char chNo, unsigned char tr, unsigned char tfmd)
{
	T8F_INIT_PARAM	param;												// timer set parameter

	param.tfmd = tfmd;
	param.trmd = T8F_TIMMODE_REPEAT;
	param.selectClk = T8F_CNTCLK_1;
	param.data = tr;		// 115200bps
	T8F_init(chNo, &param);

	T8F_setTimerRun(chNo, T8F_TIMER_RUN);
}

/*
 * start connection data send.
 * param	unsigned char  chNo      UART channel number.
 * param	unsigned char  *buf      send data
 * param	unsigned char  len       send data length
 * return	unsigned char            actual send data length
 * history	:	2009/12/25
 */
void sendUARTConnectionData(unsigned char chNo)
{
	// enable interrupt & UART
	UART_controlInt(chNo, (UART_RXERRORINT_DISABLE | UART_TXINT_ENABLE | UART_RXINT_ENABLE), UART_RXINT_1BYTE);
	UART_setComEnable(chNo, UART_CTL_ENABLE);

	// wait connection.
	flgConnectionStatus = UART_WAIT_CONNECTED;
	while(flgConnectionStatus != UART_CONNECTED) {
		if(sendWait != 0) {
			sendWait--;
		}
	}
}

/*
 * start UART data send.
 * param	unsigned char  chNo      UART channel number.
 * param	unsigned char  *buf      send data
 * param	unsigned char  len       send data length
 * return	unsigned char            actual send data length
 * history	:	2009/12/25
 */
unsigned char sendUARTData(unsigned char chNo, unsigned char *buf, unsigned char len)
{

	unsigned char	actual;												// actual send data length
	unsigned char	flag;												// check flag

	actual = 0;
	paramUart.dataTx = buf;
	paramUart.counterTx = len;

	// wait all data is sent
	while(paramUart.counterTx > 0) {
		;
	}
	while(1) {
		UART_checkReceiveFlag(chNo, &flag);
		if((flag & (UART_TX_BUSY | UART_TXBUFFER_EMPTY)) == (UART_TX_IDLE | UART_TXBUFFER_EMPTY)) {
			break;
		}
	}

	actual = len - paramUart.counterTx;

	return actual;
}

/*
 * UART interrupt handler
 * param	void
 * return	void
 * history	:	2009/12/25
 */
void uartIntHandler(void)
{
	unsigned char	flgIntr;											// interrupt flag
	unsigned char	recvData;											// recieve data

	// get interrupt status
	UART_checkReceiveFlag(UART_OSC3_MAIN_CH, &flgIntr);

	// data send
	if((flgIntr & UART_TXBUFFER_EMPTY) == UART_TXBUFFER_EMPTY) {
		// already connected
		if(flgConnectionStatus == UART_CONNECTED) {
			// buffer has data
			if(paramUart.counterTx > 0) {
				UART_setTransmitData(UART_OSC3_MAIN_CH, *paramUart.dataTx);	// send data
				paramUart.dataTx++;
				paramUart.counterTx--;
			}
		}

		// connection data
		if((flgConnectionStatus == UART_WAIT_CONNECTED) || (flgConnectionStatus == UART_CONNECTED_ACK)) {
			if(sendWait == 0) {
				UART_setTransmitData(UART_OSC3_MAIN_CH, UART_CONNECTION_REPORT);	// send report data
				if(flgConnectionStatus == UART_CONNECTED_ACK) {
					flgConnectionStatus = UART_CONNECTED;
				}
				sendWait = UART_OSC3_SENDWAIT;
			}
		}
	}

	// data receive
	if((flgIntr & UART_RXDATA_READY) == UART_RXDATA_READY) {
		UART_getReceiveData(UART_OSC3_MAIN_CH, &recvData);

		// buffer has room
		if((flgConnectionStatus != UART_NOT_CONNECTED) &&
			(paramUart.counterRx < paramUart.counterMaxRx)) {
			if(recvData < UART_CONNECTION_REPORT) {
				*paramUart.dataRx = recvData;
				paramUart.dataRx++;
				paramUart.counterRx++;
			}
		}

		// connection check
		if((flgConnectionStatus == UART_WAIT_CONNECTED) &&
			(recvData == UART_CONNECTION_REPORT)) {
				flgConnectionStatus = UART_CONNECTED_ACK;
		}
	}
}

#endif	// end of PE_UART_OSC3
