/******************************************************************
 * adc_api.h
 * 
 * ADC driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef ADC_API_H_
#define ADC_API_H_

#include <adc.h>

/******************************************
 * definition of "define"
 ******************************************/

// ADC driver return code.
#define	ADC_SUCCESS					 0								// success
#define	ADC_INVALID_PARAMETER		-1								// invalid parameter
#define	ADC_FAILURE					-2								// failure

// ADC interrupt level.
enum {
	ADC_INT_LEVEL0 = 0x00,											// 0x00:interrupt level 0
	ADC_INT_LEVEL1,													// 0x01:interrupt level 1
	ADC_INT_LEVEL2,													// 0x02:interrupt level 2
	ADC_INT_LEVEL3,													// 0x03:interrupt level 3
	ADC_INT_LEVEL4,													// 0x04:interrupt level 4
	ADC_INT_LEVEL5,													// 0x05:interrupt level 5
	ADC_INT_LEVEL6,													// 0x06:interrupt level 6
	ADC_INT_LEVEL7,													// 0x07:interrupt level 7

	ADC_INT_LEVELMAX
};

// channel number.
enum {
	ADC_CH0 = 0,													// 0x00 : channel 0
	ADC_CH1,														// 0x01 : channel 1
	ADC_CH2,														// 0x02 : channel 2
	ADC_CH3,														// 0x03 : channel 3
#ifndef	ADC_CHANNEL4
	ADC_CH4,														// 0x04 : channel 4
	ADC_CH5,														// 0x05 : channel 5
	ADC_CH6,														// 0x06 : channel 6
	ADC_CH7,														// 0x07 : channel 7
#endif	// end of ADC_CHANNEL4
	
	ADC_CH_MAX
};

// converted data store mode.
#define	ADC_STORE_MODE0				ADC_MODE0						// select store mode 0
#define	ADC_STORE_MODE1				ADC_MODE1						//					 1

// conversion mode.
#define	ADC_CONVMODE_SINGLE			ADC_SINGLE						// select conversion mode : single
#define	ADC_CONVMODE_CONTINUOUS		ADC_CONTINUOUS					//						  : continuous

// conversion trigger.
enum {
	ADC_CONVTRIG_SOFT = 0,											// 0x00 : select conversion trigger : software
	ADC_CONVTRIG_T16,												// 0x01 : 							: 16-bit timer
	ADC_CONVTRIG_RESERVED,											// 0x02 : 							: reserved
	ADC_CONVTRIG_ADTRG,												// 0x03 : 							: #ADTRG pin
	
	ADC_CONVTRIG_MAX
};

// sampling clock count.
enum {
	ADC_SAMPLING_CLOCK2 = 0,										// 0x00 : sampling clock 2 clocks
	ADC_SAMPLING_CLOCK3,											// 0x01 : 				 3 clocks
	ADC_SAMPLING_CLOCK4,											// 0x02 : 				 4 clocks
	ADC_SAMPLING_CLOCK5,											// 0x03 : 				 5 clocks
	ADC_SAMPLING_CLOCK6,											// 0x04 : 				 6 clocks
	ADC_SAMPLING_CLOCK7,											// 0x05 : 				 7 clocks
	ADC_SAMPLING_CLOCK8,											// 0x06 : 				 8 clocks
	ADC_SAMPLING_CLOCK9,											// 0x07 : 				 9 clocks
	
	ADC_SAMPLING_MAX
};

// A/D converter clock divided frequency.
enum {
	ADC_CONVCLK2 = 0,												// 0x00 : select clock divided frequency : PCLK1/2
	ADC_CONVCLK4,													// 0x01 : 								 : PCLK1/4
	ADC_CONVCLK8,													// 0x02 : 								 : PCLK1/8
	ADC_CONVCLK16,													// 0x03 : 								 : PCLK1/16
	ADC_CONVCLK32,													// 0x04 : 								 : PCLK1/32
	ADC_CONVCLK64,													// 0x05 : 								 : PCLK1/64
	ADC_CONVCLK128,													// 0x06 : 								 : PCLK1/128
	ADC_CONVCLK256,													// 0x07 : 								 : PCLK1/256
	ADC_CONVCLK512,													// 0x08 : 								 : PCLK1/512
	ADC_CONVCLK1024,												// 0x09 : 								 : PCLK1/1024
	ADC_CONVCLK2048,												// 0x0a : 								 : PCLK1/2048
	ADC_CONVCLK4096,												// 0x0b : 								 : PCLK1/4096
	ADC_CONVCLK8192,												// 0x0c : 								 : PCLK1/8192
	ADC_CONVCLK16384,												// 0x0d : 								 : PCLK1/16384
	ADC_CONVCLK32768,												// 0x0e : 								 : PCLK1/32768
	
	ADC_CONVCLK_MAX
};

// ADC enable / disable.
#define	ADC_DISABLE					ADC_BIT_DISABLE					// ADC disable
#define	ADC_ENABLE					ADC_BIT_ENABLE					//	   enable

// internal busy status.
#define	ADC_STATUS_IDLE				ADC_IDLE						// internal status : idle
#define	ADC_STATUS_BUSY				ADC_BUSY						//				   : busy

// ADC conversion control.
#define	ADC_CONTROL_STOP			ADC_STOP						// conversion control : stop
#define	ADC_CONTROL_START			ADC_START						//					  : start

// ADC interrupt.
#define	ADC_OVWINT_DISABLE			(ADOIE*ADC_BIT_DISABLE)			// overwrite interrupt disable
#define	ADC_OVWINT_ENABLE			(ADOIE*ADC_BIT_ENABLE)			//					   enable
#define	ADC_COMPINT_DISABLE			(ADCIE*ADC_BIT_DISABLE)			// conversion-complete interrupt disable
#define	ADC_COMPINT_ENABLE			(ADCIE*ADC_BIT_ENABLE)			//								 enable

// ADC interrupt flag.
#define	ADC_OVWINT_RESET			(ADOWE*ADC_RESET)				// overwrite error flag reset

#define	ADC_OVWINT_NORMAL			(ADOWE*ADC_NORMAL)				// overwrite interrupt normal
#define	ADC_OVWINT_ERROR			(ADOWE*ADC_ERROR)				//					   error
#define	ADC_COMPINT_NOTOCCURRED		(ADCF*ADC_NOTOCCURRED)			// conversion-complete interrupt not occurred
#define	ADC_COMPINT_OCCURRED		(ADCF*ADC_OCCURRED)				//								 occurred


/******************************************
 * definition of "struct"
 ******************************************/

// ADC initialize parameter.
struct tag_ADC_INIT_PARAM;
typedef struct tag_ADC_INIT_PARAM {
	unsigned short	endCh;											// select end channel
	unsigned short	startCh;										// select start channel
	unsigned short	storeMode;										// converted data store mode
	unsigned short	convMode;										// select conversion mode
	unsigned short	convTrig;										// select Kconversion trigger
	unsigned short	smpClock;										// sampling clock count
	unsigned short	convClock;										// select clock divided frequency
} ADC_INIT_PARAM;


/******************************************
 * Declarations of function prototype
 ******************************************/
extern short ADC_init(ADC_INIT_PARAM *param);
extern short ADC_setChannel(unsigned short startCh, unsigned short endCh);
extern short ADC_setStoreMode(unsigned short selectBit);
extern short ADC_setConversionMode(unsigned short selectBit);
extern short ADC_setConversionTrigger(unsigned short selectBit);
extern short ADC_setSamplingClock(unsigned short selectBit);
extern short ADC_setDividedFrequency(unsigned short selectBit);
extern short ADC_setEnable(unsigned short selectBit);
extern void ADC_getResult(unsigned short *result);
extern void ADC_getConversionChannel(unsigned short *result);
extern void ADC_checkBusyStatus(unsigned short *result);
extern short ADC_controlTrigger(unsigned short selectBit);
extern short ADC_initInt(unsigned char intLevel);
extern short ADC_controlInt(unsigned short selectBit);
extern void ADC_resetIntFlag(void);
extern void ADC_checkIntFlag(unsigned short *flag);


#endif	// end of ADC_API_H_
