/******************************************************************
 * i2cs_api.h
 * 
 * I2C slave driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef I2CS_API_H_
#define I2CS_API_H_

#include <i2cs.h>

/******************************************
 * definition of "define"
 ******************************************/

// I2C slave driver return code.
#define	I2CS_SUCCESS				 0								// success
#define	I2CS_INVALID_PARAMETER		-1								// invalid parameter
#define	I2CS_FAILURE				-2								// failure

// I2C slave interrupt level.
enum {
	I2CS_INT_LEVEL0 = 0x00,											// 0x00:interrupt level 0
	I2CS_INT_LEVEL1,												// 0x01:interrupt level 1
	I2CS_INT_LEVEL2,												// 0x02:interrupt level 2
	I2CS_INT_LEVEL3,												// 0x03:interrupt level 3
	I2CS_INT_LEVEL4,												// 0x04:interrupt level 4
	I2CS_INT_LEVEL5,												// 0x05:interrupt level 5
	I2CS_INT_LEVEL6,												// 0x06:interrupt level 6
	I2CS_INT_LEVEL7,												// 0x07:interrupt level 7

	I2CS_INT_LEVELMAX
};

// I2C slave initialize parameter.
#define	I2CS_CLKSTRETCH_OFF			I2CS_OFF						// clock stretch : off
#define	I2CS_CLKSTRETCH_ON			I2CS_ON							//				 : on
#define	I2CS_ASYNCDETCT_OFF			I2CS_OFF						// async address detection : off
#define	I2CS_ASYNCDETCT_ON			I2CS_ON							//						   : on
#define	I2CS_NOISEREMOVE_OFF		I2CS_OFF						// noise filter : off
#define	I2CS_NOISEREMOVE_ON			I2CS_ON							//				: on
#define	I2CS_BUSFREE_DISABLE		I2CS_DISABLE					// bus free request : disable
#define	I2CS_BUSFREE_ENABLE			I2CS_ENABLE						// 					: enable

// data receive response.
#define	I2CS_RESDATA_ACK			I2CS_ACK						// response ACK
#define	I2CS_RESDATA_NAK			I2CS_NAK						//			NAK

// I2C slave enable / disable.
#define	I2CS_CTL_DISABLE			I2CS_DISABLE					// I2C slave disable
#define	I2CS_CTL_ENABLE				I2CS_ENABLE						//			 enable

// I2C slave TX / RX enable / disable.
#define	I2CS_COM_DISABLE			I2CS_DISABLE					// I2C slave TX / RX disable
#define	I2CS_COM_ENABLE				I2CS_ENABLE						//			 TX / RX enable

// I2C slave interrupt.
#define	I2CS_BUSINT_DISABLE			(BSTAT_IEN*I2CS_DISABLE)		// bus status interrupt disable
#define	I2CS_BUSINT_ENABLE			(BSTAT_IEN*I2CS_ENABLE)			//						enable
#define	I2CS_RXINT_DISABLE			(RXRDY_IEN*I2CS_DISABLE)		// receive interrupt disable
#define	I2CS_RXINT_ENABLE			(RXRDY_IEN*I2CS_ENABLE)			//					 enable
#define	I2CS_TXINT_DISABLE			(TXEMP_IEN*I2CS_DISABLE)		// transmit interrupt disable
#define	I2CS_TXINT_ENABLE			(TXEMP_IEN*I2CS_ENABLE)			//					  enable

// I2C slave interrupt flag.
#define	I2CS_UDFOVF_RESET			(TXUDF*I2CS_RESET)				// data underflow / overflow flag reset
#define	I2CS_BUSFREE_RESET			(BFREQ*I2CS_RESET)				// bus free request flag reset
#define	I2CS_MISMATCH_RESET			(DMS*I2CS_RESET)				// output data mismatch flag reset
#define	I2CS_ASYNC_RESET			(ASDET*I2CS_RESET)				// async address detection status flag reset
#define	I2CS_RESPONSE_RESET			(DA_NAK*I2CS_RESET)				// NAK receive status flag reset
#define	I2CS_STOP_RESET				(DA_STOP*I2CS_RESET)			// stop condition detect flag reset

#define	I2CS_BUSSTS_UNCHANGED		(BSTAT*I2CS_UNCHANGED)			// bus status transition unchanged
#define	I2CS_BUSSTS_CHANGED			(BSTAT*I2CS_CHANGED)			//						 changed
#define	I2CS_UDFOVF_NOTOCCURRED		(TXUDF*I2CS_NOTOCCURRED)		// data underflow / overflow not occurred
#define	I2CS_UDFOVF_OCCURRED		(TXUDF*I2CS_OCCURRED)			//							 occurred
#define	I2CS_BUSFREE_NOTOCCURRED	(BFREQ*I2CS_NOTOCCURRED)		// buf free request not occurred
#define	I2CS_BUSFREE_OCCURRED		(BFREQ*I2CS_OCCURRED)			//					occurred
#define	I2CS_MISMATCH_NORMAL		(DMS*I2CS_NORMAL)				// output data mismatch normal
#define	I2CS_MISMATCH_ERROR			(DMS*I2CS_ERROR)				//						error
#define	I2CS_ASYNC_NOTDETECTED		(ASDET*I2CS_NOTDETECTED)		// async address detection status not detected
#define	I2CS_ASYNC_DETECTED			(ASDET*I2CS_DETECTED)			//								  detected
#define	I2CS_RESPONSE_ACK			(DA_NAK*I2CS_ACK)				// NAK receive status ACK
#define	I2CS_RESPONSE_NAK			(DA_NAK*I2CS_NAK)				//					  NAK
#define	I2CS_STOP_NOTDETECTED		(DA_STOP*I2CS_NOTDETECTED)		// stop condition not detected
#define	I2CS_STOP_DETECTED			(DA_STOP*I2CS_DETECTED)			//				  detected

#define	I2CS_RXDATA_NOTREADY		(RXRDY*I2CS_NOTREADY)			// receive data not ready
#define	I2CS_RXDATA_READY			(RXRDY*I2CS_READY)				//				ready
#define	I2CS_TXDATA_NOTEMPTY		(TXEMP*I2CS_NOTEMPTY)			// transmit data not empty
#define	I2CS_TXDATA_EMPTY			(TXEMP*I2CS_EMPTY)				//				 empty

// I2C slave access status.
#define	I2CS_BUSSTS_FREE			(BUSY*I2CS_FREE)				// bus status free
#define	I2CS_BUSSTS_BUSY			(BUSY*I2CS_BUSY)				//			  busy
#define	I2CS_SLAVE_NOTSELECTED		(SELECTED*I2CS_NOTSELECTED)		// I2C slave not selected
#define	I2CS_SLAVE_SELECTED			(SELECTED*I2CS_SELECTED)		//			 selected
#define	I2CS_DIRECTION_INPUT		(RW*I2CS_INPUT)					// read / write direction input
#define	I2CS_DIRECTION_OUTPUT		(RW*I2CS_OUTPUT)				//						  output


/******************************************
 * Declarations of function prototype
 ******************************************/
extern void I2CS_reset(void);
extern short I2CS_setAddress(unsigned char address);
extern short I2CS_setClockStretch(unsigned short selectBit);
extern short I2CS_setAsyncDetection(unsigned short selectBit);
extern short I2CS_setNoiseRemove(unsigned short selectBit);
extern short I2CS_setBusFreeReq(unsigned short selectBit);
extern short I2CS_setReceiveResponse(unsigned short selectBit);
extern short I2CS_init(unsigned char address, unsigned short noise, unsigned short stretch);
extern short I2CS_setEnable(unsigned short selectBit);
extern short I2CS_setComEnable(unsigned short selectBit);
extern void I2CS_setTransmitData(unsigned char data);
extern void I2CS_getReceiveData(unsigned char *data);
extern short I2CS_initInt(unsigned char intLevel);
extern short I2CS_controlInt(unsigned short selectBit);
extern short I2CS_resetIntFlag(unsigned short selectBit);
extern void I2CS_checkBusStatusIntFlag(unsigned short *flag);
extern void I2CS_checkIntFlag(unsigned short *flag);
extern void I2CS_checkAccessStatus(unsigned short *flag);


#endif	// end of I2CS_API_H_
