/******************************************************************
 * t16e_api.h
 * 
 * PWM timer driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef T16E_API_H_
#define T16E_API_H_

#include <t16e.h>

/******************************************
 * definition of "define"
 ******************************************/

// PWM timer driver return code.
#define	T16E_SUCCESS				 0								// success
#define	T16E_INVALID_PARAMETER		-1								// invalid parameter
#define	T16E_FAILURE				-2								// failure

// PWM timer interrupt level.
enum {
	T16E_INT_LEVEL0 = 0x00,											// 0x00:interrupt level 0
	T16E_INT_LEVEL1,												// 0x01:interrupt level 1
	T16E_INT_LEVEL2,												// 0x02:interrupt level 2
	T16E_INT_LEVEL3,												// 0x03:interrupt level 3
	T16E_INT_LEVEL4,												// 0x04:interrupt level 4
	T16E_INT_LEVEL5,												// 0x05:interrupt level 5
	T16E_INT_LEVEL6,												// 0x06:interrupt level 6
	T16E_INT_LEVEL7,												// 0x07:interrupt level 7

	T16E_INT_LEVELMAX
};

// channel number.
enum {
	T16E_CH0 = 0,													// 0x00 : channel 0
#ifdef	T16E_CHANNEL2
	T16E_CH1,														// 0x01 : channel 1
#endif	// end of T16E_CHANNEL2
	
	T16E_CH_MAX
};

// PWM timer mode set parameter.
#define	T16E_TIMMODE_TOUT3L			T16E_LOW						// select initial output level : TOUT3=Low / TOUTN3=High
#define	T16E_TIMMODE_TOUT3H			T16E_HIGH						//							   : TOUT3=High / TOUTN3=Low

#define	T16E_TIMMODE_CLK_NORMAL		T16E_NORMAL						// select fine mode : normal mode
#define	T16E_TIMMODE_CLK_FINEMODE	T16E_FINE						//					: fine mode

#define	T16E_TIMMODE_CMP_DISABLE	T16E_DISABLE					// comparison buffer disable
#define	T16E_TIMMODE_CMP_ENABLE		T16E_ENABLE						//					 enable

#define	T16E_TIMMODE_NORMAL			T16E_NORMAL						// select inverse output control : normal
#define	T16E_TIMMODE_INVERSE		T16E_INVERT						//								 : invert

#define	T16E_TIMMODE_INTERNAL		T16E_INTERNAL					// select input clock : internal
#define	T16E_TIMMODE_EXTERNAL		T16E_EXTERNAL					//					  : external

#define	T16E_TIMMODE_DISABLE		T16E_DISABLE					// clock output disable
#define	T16E_TIMMODE_ENABLE			T16E_ENABLE						//				enable

// PWM timer run / stop control.
#define	T16E_TIMER_STOP				T16E_STOP						// timer stop
#define	T16E_TIMER_RUN				T16E_RUN						//		 run

// PWM timer input clock.
enum {
	T16E_CNTCLK_1 = 0,												// 0x00 : select timer input clock : PCLK1/1
	T16E_CNTCLK_2,													// 0x01 :						   : PCLK1/2
	T16E_CNTCLK_4,													// 0x02 :						   : PCLK1/4
	T16E_CNTCLK_8,													// 0x03 :						   : PCLK1/8
	T16E_CNTCLK_16,													// 0x04 :						   : PCLK1/16
	T16E_CNTCLK_32,													// 0x05 :						   : PCLK1/32
	T16E_CNTCLK_64,													// 0x06 :						   : PCLK1/64
	T16E_CNTCLK_128,												// 0x07 :						   : PCLK1/128
	T16E_CNTCLK_256,												// 0x08 :						   : PCLK1/256
	T16E_CNTCLK_512,												// 0x09 :						   : PCLK1/512
	T16E_CNTCLK_1024,												// 0x0A :						   : PCLK1/1024
	T16E_CNTCLK_2048,												// 0x0B :						   : PCLK1/2048
	T16E_CNTCLK_4096,												// 0x0C :						   : PCLK1/4096
	T16E_CNTCLK_8192,												// 0x0D :						   : PCLK1/8192
	T16E_CNTCLK_16384,												// 0x0E :						   : PCLK1/16384
	
	T16E_CNTCLK_MAX
};

// PWM timer interrupt.
#define	T16E_CMPBINT_DISABLE		(CBIE*T16E_DISABLE)				// compare B interrupt disable
#define	T16E_CMPBINT_ENABLE			(CBIE*T16E_ENABLE)				//					   enable
#define	T16E_CMPAINT_DISABLE		(CAIE*T16E_DISABLE)				// compare A interrupt disable
#define	T16E_CMPAINT_ENABLE			(CAIE*T16E_ENABLE)				//					   enable

// PWM timer interrupt flag.
#define	T16E_BINT_RESET				(CBIF*T16E_RESET)				// compare B flag reset
#define	T16E_AINT_RESET				(CAIF*T16E_RESET)				// compare A flag reset

#define	T16E_BINT_NOTOCCURRED		(CBIF*T16E_NOTOCCURRED)			// not occurred
#define	T16E_BINT_OCCURRED			(CBIF*T16E_OCCURRED)			// occurred
#define	T16E_AINT_NOTOCCURRED		(CAIF*T16E_NOTOCCURRED)			// not occurred
#define	T16E_AINT_OCCURRED			(CAIF*T16E_OCCURRED)			// occurred


/******************************************
 * definition of "struct"
 ******************************************/

// PWM timer mode set parameter.
struct tag_T16E_TIM_MODE;
typedef struct tag_T16E_TIM_MODE {
	unsigned short	initol;											// select initial output level
	unsigned short	selfm;											// select fine mode
	unsigned short	cbufen;											// comparison buffer enable
	unsigned short	invout;											// select inverse output control
	unsigned short	clksel;											// select input clock
	unsigned short	outen;											// clock output enabl
} T16E_TIM_MODE;

//PWM timer initialize parameter.
struct tag_T16E_INIT_PARAM;
typedef struct tag_T16E_INIT_PARAM {
	unsigned short	dataA;											// compare data A
	unsigned short	dataB;											// compare data B
	unsigned short	selectClk;										// Timer Input Clock Select
	unsigned short	data;											// counter data
} T16E_INIT_PARAM;


/******************************************
 * Declarations of function prototype
 ******************************************/
extern short T16E_init(unsigned char chNo, T16E_TIM_MODE *mode, T16E_INIT_PARAM *param);
extern short T16E_setCompareData(unsigned char chNo, unsigned short dataA, unsigned short dataB);
extern short T16E_getCounterData(unsigned char chNo, unsigned short *data);
extern short T16E_setTimerMode(unsigned char chNo, T16E_TIM_MODE *mode);
extern short T16E_resetTimer(unsigned char chNo);
extern short T16E_setTimerRun(unsigned char chNo, unsigned short selectBit);
extern short T16E_setInputClock(unsigned char chNo, unsigned short selectBit);
extern short T16E_initInt(unsigned char chNo, unsigned char intLevel);
extern short T16E_controlInt(unsigned char chNo, unsigned short selectBit);
extern short T16E_resetIntFlag(unsigned char chNo, unsigned short selectBit);
extern short T16E_checkIntFlag(unsigned char chNo, unsigned short *flag);


#endif	// end of T16E_API_H_
