/******************************************************************
 * t8f_api.h
 * 
 * 8-bit timer driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef T8F_API_H_
#define T8F_API_H_

#include <t8f.h>

/******************************************
 * definition of "define"
 ******************************************/

// 8-bit timer driver return code.
#define	T8F_SUCCESS					 0								// success
#define	T8F_INVALID_PARAMETER		-1								// invalid parameter
#define	T8F_FAILURE					-2								// failure

// 8-bit timer interrupt level.
enum {
	T8F_INT_LEVEL0 = 0x00,											// 0x00:interrupt level 0
	T8F_INT_LEVEL1,													// 0x01:interrupt level 1
	T8F_INT_LEVEL2,													// 0x02:interrupt level 2
	T8F_INT_LEVEL3,													// 0x03:interrupt level 3
	T8F_INT_LEVEL4,													// 0x04:interrupt level 4
	T8F_INT_LEVEL5,													// 0x05:interrupt level 5
	T8F_INT_LEVEL6,													// 0x06:interrupt level 6
	T8F_INT_LEVEL7,													// 0x07:interrupt level 7

	T8F_INT_LEVELMAX
};

// channel number.
enum {
	T8F_CH0 = 0,													// 0x00 : channel 0
#ifdef	T8F_CHANNEL2
	T8F_CH1,														// 0x01 : channel 1
#endif	// end of T8F_CHANNEL2
	
	T8F_CH_MAX
};

// 8-bit timer input clock.
enum {
	T8F_CNTCLK_1 = 0,												// 0x00 : select timer input clock : PCLK1/1
	T8F_CNTCLK_2,													// 0x01 :						   : PCLK1/2
	T8F_CNTCLK_4,													// 0x02 :						   : PCLK1/4
	T8F_CNTCLK_8,													// 0x03 :						   : PCLK1/8
	T8F_CNTCLK_16,													// 0x04 :						   : PCLK1/16
	T8F_CNTCLK_32,													// 0x05 :						   : PCLK1/32
	T8F_CNTCLK_64,													// 0x06 :						   : PCLK1/64
	T8F_CNTCLK_128,													// 0x07 :						   : PCLK1/128
	T8F_CNTCLK_256,													// 0x08 :						   : PCLK1/256
	T8F_CNTCLK_512,													// 0x09 :						   : PCLK1/512
	T8F_CNTCLK_1024,												// 0x0A :						   : PCLK1/1024
	T8F_CNTCLK_2048,												// 0x0B :						   : PCLK1/2048
	T8F_CNTCLK_4096,												// 0x0C :						   : PCLK1/4096
	T8F_CNTCLK_8192,												// 0x0D :						   : PCLK1/8192
	T8F_CNTCLK_16384,												// 0x0E :						   : PCLK1/16384
	
	T8F_CNTCLK_MAX
};

// 8-bit timer mode set parameter.
enum {
	T8F_TIMMODE_PATTERN0 = 0,										// 0x00 : setup fine mode : pattern 0
	T8F_TIMMODE_PATTERN1,											// 0x01 : 				  : pattern 1
	T8F_TIMMODE_PATTERN2,											// 0x02 : 				  : pattern 2
	T8F_TIMMODE_PATTERN3,											// 0x03 : 				  : pattern 3
	T8F_TIMMODE_PATTERN4,											// 0x04 : 				  : pattern 4
	T8F_TIMMODE_PATTERN5,											// 0x05 : 				  : pattern 5
	T8F_TIMMODE_PATTERN6,											// 0x06 : 				  : pattern 6
	T8F_TIMMODE_PATTERN7,											// 0x07 : 				  : pattern 7
	T8F_TIMMODE_PATTERN8,											// 0x08 : 				  : pattern 8
	T8F_TIMMODE_PATTERN9,											// 0x09 : 				  : pattern 9
	T8F_TIMMODE_PATTERNA,											// 0x0a : 				  : pattern A
	T8F_TIMMODE_PATTERNB,											// 0x0b : 				  : pattern B
	T8F_TIMMODE_PATTERNC,											// 0x0c : 				  : pattern C
	T8F_TIMMODE_PATTERND,											// 0x0d : 				  : pattern D
	T8F_TIMMODE_PATTERNE,											// 0x0e : 				  : pattern E
	T8F_TIMMODE_PATTERNF,											// 0x0f : 				  : pattern F
	
	T8F_TIMMODE_MAX
};

#define	T8F_TIMMODE_REPEAT			T8F_REPEAT						// select count mode : repeat
#define	T8F_TIMMODE_ONESHOT			T8F_ONESHOT						//					 : one shot

// timer run / stop control.
#define	T8F_TIMER_STOP				T8F_STOP						// timer stop
#define	T8F_TIMER_RUN				T8F_RUN							//		 run

// 8-bit timer interrupt.
#define	T8F_INT_DISABLE				T8F_DISABLE						// interrupt disable
#define	T8F_INT_ENABLE				T8F_ENABLE						//			 enable

// 8-bit timer interrupt flag.
#define	T8F_TIMERINT_NOTOCCURRED	T8F_NOTOCCURRED					// not occurred
#define	T8F_TIMERINT_OCCURRED		T8F_OCCURRED					// occurred


/******************************************
 * definition of "struct"
 ******************************************/

// 8-bit timer initialize parameter.
struct tag_T8F_INIT_PARAM;
typedef struct tag_T8F_INIT_PARAM {
	unsigned short	tfmd;											// Fine Mode Setup
	unsigned short	trmd;											// Count Mode Select
	unsigned short	selectClk;										// Timer Input Clock Select
	unsigned short	data;											// 8-bit Timer Reload Data
} T8F_INIT_PARAM;


/******************************************
 * Declarations of function prototype
 ******************************************/
extern short T8F_init(unsigned char chNo, T8F_INIT_PARAM *param);
extern short T8F_setInputClock(unsigned char chNo, unsigned short selectBit);
extern short T8F_setReloadData(unsigned char chNo, unsigned char data);
extern short T8F_getCounterData(unsigned char chNo, unsigned char *data);
extern short T8F_setTimerMode(unsigned char chNo, unsigned short tfmd, unsigned short trmd);
extern short T8F_resetTimer(unsigned char chNo);
extern short T8F_setTimerRun(unsigned char chNo, unsigned short selectBit);
extern short T8F_initInt(unsigned char intLevel);
extern short T8F_controlInt(unsigned char chNo, unsigned short selectBit);
extern short T8F_resetIntFlag(unsigned char chNo);
extern short T8F_checkIntFlag(unsigned char chNo, unsigned short *flag);


#endif	// end of T8F_API_H_
