/******************************************************************
 * i2cs.h
 * 
 * I2C slave driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef I2CS_H_
#define I2CS_H_


#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct i2cs_trnsBitF {
	unsigned int	sdata	: 8;
	unsigned int			: 8;
};

struct i2cs_recvBitF {
	unsigned int	rdata	: 8;
	unsigned int			: 8;
};

struct i2cs_sadrsBitF {
	unsigned int	sadrs	: 7;
	unsigned int			: 9;
};

struct i2cs_ctlBitF {
	unsigned int	com_mode	: 1;
	unsigned int	asdet_en	: 1;
	unsigned int	nf_en		: 1;
	unsigned int	clkstr_en	: 1;
	unsigned int	bfreq_en	: 1;
	unsigned int	nak_ans		: 1;
	unsigned int	softreset	: 1;
	unsigned int	i2c_en		: 1;
	unsigned int	tbuf_clr	: 1;
	unsigned int				: 7;
};

struct i2cs_statBitF {
	unsigned int	da_stop		: 1;
	unsigned int	da_nak		: 1;
	unsigned int	asdet		: 1;
	unsigned int	dms			: 1;
	unsigned int	bfreq		: 1;
	unsigned int	tr_udovf	: 1;
	unsigned int				: 1;
	unsigned int	bstat		: 1;
	unsigned int				: 8;
};

struct i2cs_astatBitF {
	unsigned int	rw			: 1;
	unsigned int	selected	: 1;
	unsigned int	busy		: 1;
	unsigned int	txemp		: 1;
	unsigned int	rxrdy		: 1;
	unsigned int				: 11;
};

struct i2cs_ictlBitF {
	unsigned int	bstat_ien	: 1;
	unsigned int	rxrdy_ien	: 1;
	unsigned int	txemp_ien	: 1;
	unsigned int				: 13;
};
#endif	// end of BIT_FIELD


/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define	I2CS_BASE_ADDR						(BASE_ADDR+I2CS_OFFSET)


/***********************************************
 ****** The Definition of I2C ******************
 ****** [I2C]                 ******************
 ***********************************************/
// I2C Slave Transmit Data Register
#define	R16_I2CS_TRNS						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x00)
	#define	SDATA_SFT							0
	#define	SDATA								BIT(0x00ff, SDATA_SFT)		// transmit data

#ifdef	BIT_FIELD
#define I2CS_TRNS							*(volatile unsigned short *)(BASE_ADDR+0x0360)
#define BF_I2CS_TRNS						(*(volatile struct i2cs_trnsBitF *)(BASE_ADDR+0x0360))
	#define I2CS_TRNS_SDATA						BF_I2CS_TRNS.sdata			// transmit data
#endif	// end of BIT_FIELD


// I2C Slave Receive Data Register
#define	R16_I2CS_RECV						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x02)
	#define	RDATA_SFT							0
	#define	RDATA								BIT(0x00ff, RDATA_SFT)		// Receive data

#ifdef	BIT_FIELD
#define I2CS_RECV							*(volatile unsigned short *)(BASE_ADDR+0x0362)
#define BF_I2CS_RECV						(*(volatile struct i2cs_recvBitF *)(BASE_ADDR+0x0362))
	#define I2CS_RECV_RDATA						BF_I2CS_RECV.rdata			// Receive data
#endif	// end of BIT_FIELD


// I2C Slave Address Setup Register
#define	R16_I2CS_SADRS						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x04)
	#define	SADRS_SFT							0
	#define	SADRS								BIT(0x007f, SADRS_SFT)		// Slave address

#ifdef	BIT_FIELD
#define I2CS_SADRS							*(volatile unsigned short *)(BASE_ADDR+0x0364)
#define BF_I2CS_SADRS						(*(volatile struct i2cs_sadrsBitF *)(BASE_ADDR+0x0364))
	#define I2CS_SADRS_SADRS					BF_I2CS_SADRS.sadrs			// Slave address
#endif	// end of BIT_FIELD


// I2C Slave Control Register
#define	R16_I2CS_CTL						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x06)
	#define	TBUF_CLR_SFT						8
	#define	I2C_EN_SFT							7
	#define	SOFTRESET_SFT						6
	#define	NAK_ANS_SFT							5
	#define	BFREQ_EN_SFT						4
	#define	CLKSTR_EN_SFT						3
	#define	NF_EN_SFT							2
	#define	ASDET_EN_SFT						1
	#define	COM_MODE_SFT						0
	#define	TBUF_CLR							BIT(0x0001, TBUF_CLR_SFT)	// I2CS Trans register clear
	#define	I2C_EN								BIT(0x0001, I2C_EN_SFT)		// I2C Slave Enable
		#define	I2CS_DISABLE						0x0000						// disable
		#define	I2CS_ENABLE							0x0001						// enable
	#define	SOFTRESET							BIT(0x0001, SOFTRESET_SFT)	// Software reset
		#define	SOFTRESET_CANCEL					(SOFTRESET*0x00)			// cancel
		#define	SOFTRESET_RESET						(SOFTRESET*0x01)			// reset
	#define	NAK_ANS								BIT(0x0001, NAK_ANS_SFT)	// NAK Answer
		#define	I2CS_ACK							0x0000						// ACK
		#define	I2CS_NAK							0x0001						// NAK
	#define	BFREQ_EN							BIT(0x0001, BFREQ_EN_SFT)	// Bus Free Request enable
	#define	CLKSTR_EN							BIT(0x0001, CLKSTR_EN_SFT)	// Clock Stretch On/Off
		#define	I2CS_OFF							0x0000						// off
		#define	I2CS_ON								0x0001						// on
	#define	NF_EN								BIT(0x0001, NF_EN_SFT)		// Noise Filter On/Off
	#define	ASDET_EN							BIT(0x0001, ASDET_EN_SFT)	// Async. Address Detection On/Off
	#define	COM_MODE							BIT(0x0001, COM_MODE_SFT)	// I2C Slave Communication Mode

#ifdef	BIT_FIELD
#define I2CS_CTL							*(volatile unsigned short *)(BASE_ADDR+0x0366)
#define BF_I2CS_CTL							(*(volatile struct i2cs_ctlBitF *)(BASE_ADDR+0x0366))
	#define I2CS_CTL_TBUF_CLR					BF_I2CS_CTL.tbuf_clr		// I2C Trans register clear
	#define I2CS_CTL_I2C_EN						BF_I2CS_CTL.i2c_en			// I2C Device Enable
	#define I2CS_CTL_SOFTRESET					BF_I2CS_CTL.softreset		// Soft reset enable
	#define I2CS_CTL_NAK_ANS					BF_I2CS_CTL.nak_ans			// Receiving response
	#define I2CS_CTL_BFREQ_EN					BF_I2CS_CTL.bfreq_en		// bus free request enable
	#define I2CS_CTL_CLKSTR_EN					BF_I2CS_CTL.clkstr_en		// Clock stretch enable
	#define I2CS_CTL_NF_EN						BF_I2CS_CTL.nf_en			// Noise remove on/off
	#define I2CS_CTL_ASDET_EN					BF_I2CS_CTL.asdet_en		// Asynchronous address detection enable
	#define I2CS_CTL_COM_MODE					BF_I2CS_CTL.com_mode		// Active/Standby
#endif	// end of BIT_FIELD


// I2C Slave Status Register
// NOTE : The writing that used the bit field inhibits.
#define	R16_I2CS_STAT						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x08)
	#define	BSTAT_SFT							7
	#define	TXUDF_SFT							5
	#define	RXOVF_SFT							5
	#define	BFREQ_SFT							4
	#define	DMS_SFT								3
	#define	ASDET_SFT							2
	#define	DA_NAK_SFT							1
	#define	DA_STOP_SFT							0
	#define	BSTAT								BIT(0x0001, BSTAT_SFT)		// Bus Status Transition
		#define	I2CS_UNCHANGED						0x0000						// unchanged
		#define	I2CS_CHANGED						0x0001						// changed
	#define	TXUDF								BIT(0x0001, TXUDF_SFT)		// Transmit Data Underflow
	#define	RXOVF								BIT(0x0001, RXOVF_SFT)		// Receive Data Overflow
		#define	I2CS_INVALID						0x0000						// invalid
		#define	I2CS_RESET							0x0001						// reset
		#define	I2CS_NOTOCCURRED					0x0000						// not occurred
		#define	I2CS_OCCURRED						0x0001						// occurred
	#define	BFREQ								BIT(0x0001, BFREQ_SFT)		// Bus Free Request
	#define	DMS									BIT(0x0001, DMS_SFT)		// Output Data Mismatch
		#define	I2CS_NORMAL							0x0000						// normal
		#define	I2CS_ERROR							0x0001						// error
	#define	ASDET								BIT(0x0001, ASDET_SFT)		// Async. Address Detection Status
		#define	I2CS_NOTDETECTED					0x0000						// notdetected
		#define	I2CS_DETECTED						0x0001						// detected
	#define	DA_NAK								BIT(0x0001, DA_NAK_SFT)		// NAK Receive Status
	#define	DA_STOP								BIT(0x0001, DA_STOP_SFT)	// Stop Condition Detect
		#define	I2CS_BUSINT_MASK					(TXUDF | BFREQ | DMS | ASDET | DA_NAK | DA_STOP)
																			// bus status interrupt mask data

#ifdef	BIT_FIELD
#define I2CS_STAT							*(volatile unsigned short *)(BASE_ADDR+0x0368)
#define BF_I2CS_STAT						(*(volatile struct i2cs_statBitF *)(BASE_ADDR+0x0368))
	#define I2CS_STAT_BSTAT						BF_I2CS_STAT.bstat			// Change notice of bus condition
	#define I2CS_STAT_TXUDF_RXOVF				BF_I2CS_STAT.tr_udovf		// transmit buffer under flow
																			// Received buffer over flow
	#define I2CS_STAT_BFREQ						BF_I2CS_STAT.bfreq			// bus free request
	#define I2CS_STAT_DMS						BF_I2CS_STAT.dms			// Output data mismatch
	#define I2CS_STAT_ASDET						BF_I2CS_STAT.asdet			// Asynchronous address detection
	#define I2CS_STAT_DA_NAK					BF_I2CS_STAT.da_nak			// transmit data response
	#define I2CS_STAT_DA_STOP					BF_I2CS_STAT.da_stop		// stop condition detection
#endif	// end of BIT_FIELD


// I2C Slave Access Status Register
#define	R16_I2CS_ASTAT						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x0a)
	#define	RXRDY_SFT							4
	#define	TXEMP_SFT							3
	#define	BUSY_SFT							2
	#define	SELECTED_SFT						1
	#define	RW_SFT								0
	#define	RXRDY								BIT(0x0001, RXRDY_SFT)		// Receive Data Ready
		#define	I2CS_NOTREADY						0x0000						// notready
		#define	I2CS_READY							0x0001						// ready
	#define	TXEMP								BIT(0x0001, TXEMP_SFT)		// Transmit Data Empty
		#define	I2CS_NOTEMPTY						0x0000						// notempty
		#define	I2CS_EMPTY							0x0001						// empty
		#define	I2CS_INT_MASK						(RXRDY | TXEMP)				// interrupt mask data
	#define	BUSY								BIT(0x0001, BUSY_SFT)		// I2C Bus Status
		#define	I2CS_FREE							0x0000						// free
		#define	I2CS_BUSY							0x0001						// busy
	#define	SELECTED							BIT(0x0001, SELECTED_SFT)	// I2C Slave Select Status
		#define	I2CS_NOTSELECTED					0x0000						// not selected
		#define	I2CS_SELECTED						0x0001						// selected
	#define	RW									BIT(0x0001, RW_SFT)			// Read/Write Direction
		#define	I2CS_INPUT							0x0000						// input
		#define	I2CS_OUTPUT							0x0001						// output
		#define	I2CS_STATUS_MASK					(BUSY | SELECTED | RW)		// status mask data

#ifdef	BIT_FIELD
#define I2CS_ASTAT							*(volatile unsigned short *)(BASE_ADDR+0x036a)
#define BF_I2CS_ASTAT						(*(volatile struct i2cs_astatBitF *)(BASE_ADDR+0x036a))
	#define I2CS_ASTAT_RXRDY					BF_I2CS_ASTAT.rxrdy			// Recieved buffer ready
	#define I2CS_ASTAT_TXEMP					BF_I2CS_ASTAT.txemp			// transmit buffer ready
	#define I2CS_ASTAT_BUSY						BF_I2CS_ASTAT.busy			// situation of bus
	#define I2CS_ASTAT_SELECTED					BF_I2CS_ASTAT.selected		// Selecting from master
	#define I2CS_ASTAT_RW						BF_I2CS_ASTAT.rw			// R/W request of master
#endif	// end of BIT_FIELD


// I2C Slave Interrupt Control Register
#define	R16_I2CS_ICTL						(volatile unsigned short *)(unsigned long)(I2CS_BASE_ADDR+0x0c)
	#define	BSTAT_IEN_SFT						2
	#define	RXRDY_IEN_SFT						1
	#define	TXEMP_IEN_SFT						0
	#define	BSTAT_IEN							BIT(0x0001, BSTAT_IEN_SFT)	// Bus Status Interrupt Enable
	#define	RXRDY_IEN							BIT(0x0001, RXRDY_IEN_SFT)	// Receive Interrupt Enable
	#define	TXEMP_IEN							BIT(0x0001, TXEMP_IEN_SFT)	// Transmit Interrupt Enable
		#define	I2CS_IEN_MASK						(BSTAT_IEN | RXRDY_IEN | TXEMP_IEN)
																			// interrupt enable mask data

#ifdef	BIT_FIELD
#define I2CS_ICTL							*(volatile unsigned short *)(BASE_ADDR+0x036c)
#define BF_I2CS_ICTL						(*(volatile struct i2cs_ictlBitF *)(BASE_ADDR+0x036c))
	#define I2CS_ICTL_BSTAT_IEN					BF_I2CS_ICTL.txemp_ien		// BSTAT interrupt enable
	#define I2CS_ICTL_RXRDY_IEN					BF_I2CS_ICTL.rxrdy_ien		// RXRDY interrupt enable
	#define I2CS_ICTL_TXEMP_IEN					BF_I2CS_ICTL.bstat_ien		// TXEMP interrupt enable
#endif	// end of BIT_FIELD


#endif	// end of I2CS_H_
