/******************************************************************
 * lcd.h
 * 
 * LCD driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef LCD_H_
#define LCD_H_


#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct lcd_dctlBitF {
	unsigned int	dspc	: 2;
	unsigned int			: 2;
	unsigned int	dsprev	: 1;
	unsigned int	dspar	: 1;
	unsigned int	comrev	: 1;
	unsigned int	segrev	: 1;
};

struct lcd_cadjBitF {
	unsigned int	lc		: 4;
	unsigned int			: 4;
};

struct lcd_cctlBitF {
	unsigned int	lduty	: 3;
	unsigned int			: 2;
	unsigned int	lfrout	: 1;
	unsigned int	frmcnt	: 2;
};

struct lcd_vregBitF {
	unsigned int	vcsel	: 1;
	unsigned int			: 3;
	unsigned int	hvld	: 1;
	unsigned int			: 3;
};

struct lcd_imskBitF {
	unsigned int	frmie	: 1;
	unsigned int			: 7;
};

struct lcd_iflgBitF {
	unsigned int	frmif	: 1;
	unsigned int			: 7;
};
#endif	// end of BIT_FIELD


/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define	LCD_BASE_ADDR						(BASE_ADDR+LCD_OFFSET)


/***********************************************
 ****** The Definition of LCD Driver ***********
 ****** [LCD]                        ***********
 ***********************************************/
// LCD Display Control Register
#define	R08_LCD_DCTL						(volatile unsigned char *)(unsigned long)(LCD_BASE_ADDR+0x00)
	#define	SEGREV_SFT							7
	#define	COMREV_SFT							6
	#define	DSPAR_SFT							5
	#define	DSPREV_SFT							4
	#define	DSPC_SFT							0
	#define	SEGREV								BIT(0x01, SEGREV_SFT)		// Segment Output Assignment Control
		#define	LCD_REVERSE							0x00						// reverse
		#define	LCD_NORMAL							0x01						// normal
	#define	COMREV								BIT(0x01, COMREV_SFT)		// Common Output Assignment Control
	#define	DSPAR								BIT(0x01, DSPAR_SFT)		// Display Memory Area Control
		#define	LCD_AREA0							0x00						// area 0
		#define	LCD_AREA1							0x01						// area 1
	#define	DSPREV								BIT(0x01, DSPREV_SFT)		// Reverse Display Control
	#define	DSPC								BIT(0x03, DSPC_SFT)			// LCD Display Control

#ifdef	BIT_FIELD
#define LCD_DCTL							*(volatile unsigned char *)(BASE_ADDR+0x10a0)
#define BF_LCD_DCTL							(*(volatile struct lcd_dctlBitF *)(BASE_ADDR+0x10a0))
	#define LCD_DCTL_SEGREV						BF_LCD_DCTL.segrev			// Segment output assignment control
	#define LCD_DCTL_COMREV						BF_LCD_DCTL.comrev			// Common output assignment control
	#define LCD_DCTL_DSPAR						BF_LCD_DCTL.dspar			// Display memory area control
	#define LCD_DCTL_DSPREV						BF_LCD_DCTL.dsprev			// Reverse display control
	#define LCD_DCTL_DSPC						BF_LCD_DCTL.dspc			// LDC display control
#endif	// end of BIT_FIELD


// LCD Contrast Adjust Register
#define	R08_LCD_CADJ						(volatile unsigned char *)(unsigned long)(LCD_BASE_ADDR+0x01)
	#define	LC_SFT								0
	#define	LC									BIT(0x0f, LC_SFT)			// LCD Contrast Adjustment

#ifdef	BIT_FIELD
#define LCD_CADJ							*(volatile unsigned char *)(BASE_ADDR+0x10a1)
#define BF_LCD_CADJ							(*(volatile struct lcd_cadjBitF *)(BASE_ADDR+0x10a1))
	#define LCD_CADJ_LC							BF_LCD_CADJ.lc				// LCD contrast adjustment
#endif	// end of BIT_FIELD


// LCD Clock Control Register
#define	R08_LCD_CCTL						(volatile unsigned char *)(unsigned long)(LCD_BASE_ADDR+0x02)
	#define	FRMCNT_SFT							6
	#define	LFROUT_SFT							5
	#define	LDUTY_SFT							0
	#define	FRMCNT								BIT(0x03, FRMCNT_SFT)		// Frame Frequency Control
	#define	LFROUT								BIT(0x01, LFROUT_SFT)		// LFR Output Control
		#define	LCD_OFF								0x00						// off
		#define	LCD_ON								0x01						// on
	#define	LDUTY								BIT(0x07, LDUTY_SFT)		// LCD Duty Select

#ifdef	BIT_FIELD
#define LCD_CCTL							*(volatile unsigned char *)(BASE_ADDR+0x10a2)
#define BF_LCD_CCTL							(*(volatile struct lcd_cctlBitF *)(BASE_ADDR+0x10a2))
	#define LCD_CCTL_FRMCNT						BF_LCD_CCTL.frmcnt			// Frame freq control
	#define LCD_CCTL_LFROUT						BF_LCD_CCTL.lfrout			// Frame signal output control
	#define LCD_CCTL_LDUTY						BF_LCD_CCTL.lduty			// LCD duty select
#endif	// end of BIT_FIELD


// LCD Voltage Regulator Control Register
#define	R08_LCD_VREG						(volatile unsigned char *)(unsigned long)(LCD_BASE_ADDR+0x03)
	#define	HVLD_SFT							4
	#define	VCSEL_SFT							0
	#define	HVLD								BIT(0x01, HVLD_SFT)			// LCD heavy load protection mode
	#define	VCSEL								BIT(0x01, VCSEL_SFT)		// Power source select for LCD Voltage regulator
		#define	LCD_1V								0x00						// 1V
		#define	LCD_2V								0x01						// 2V

#ifdef	BIT_FIELD
#define LCD_VREG							*(volatile unsigned char *)(BASE_ADDR+0x10a3)
#define BF_LCD_VREG							(*(volatile struct lcd_vregBitF *)(BASE_ADDR+0x10a3))
	#define LCD_VREG_LHVLD						BF_LCD_VREG.hvld			// LCD heavy load protection mode
	#define LCD_VREG_VCSEL						BF_LCD_VREG.vcsel			// Power source select for LCD Voltage regulator
#endif	// end of BIT_FIELD


// LCD Interrupt Mask Register
#define	R08_LCD_IMSK						(volatile unsigned char *)(unsigned long)(LCD_BASE_ADDR+0x05)
	#define	FRMIE_SFT							0
	#define	FRMIE								BIT(0x01, FRMIE_SFT)		// Frame Signal Interrupt Enable
		#define	LCD_DISABLE							0x00						// disable
		#define	LCD_ENABLE							0x01						// enable

#ifdef	BIT_FIELD
#define LCD_IMSK							*(volatile unsigned char *)(BASE_ADDR+0x10a5)
#define BF_LCD_IMSK							(*(volatile struct lcd_imskBitF *)(BASE_ADDR+0x10a5))
	#define LCD_IMSK_FRMIE						BF_LCD_IMSK.frmie			// Frame signal interrupt enable
#endif	// end of BIT_FIELD


// LCD Interrupt Flag Register
#define	R08_LCD_IFLG						(volatile unsigned char *)(unsigned long)(LCD_BASE_ADDR+0x06)
	#define	FRMIF_SFT							0
	#define	FRMIF								BIT(0x01, FRMIF_SFT)		// Frame Signal Interrupt
		#define	LCD_INVALID							0x00						// invalid
		#define	LCD_RESET							0x01						// reset
		#define	LCD_NOTOCCURRED						0x00						// not occurred
		#define	LCD_OCCURRED						0x01						// occurred

#ifdef	BIT_FIELD
#define LCD_IFLG							*(volatile unsigned char *)(BASE_ADDR+0x10a6)
#define BF_LCD_IFLG							(*(volatile struct lcd_iflgBitF *)(BASE_ADDR+0x10a6))
	#define LCD_IFLG_FRMIF						BF_LCD_IFLG.frmif			// Frame signal interrupt flag  (read only)
#endif	// end of BIT_FIELD


// LCD display RAM area
#define	R08_LCD_RAM							(volatile unsigned char *)(unsigned long)(LCD_RAM_ADDR)


#endif	// end of LCD_H_
