/******************************************************************
 * osc.h
 *
 * OSC driver
 *
 * Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 *****************************************************************/

#ifndef OSC_H_
#define OSC_H_


#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct osc_srcBitF {
	unsigned int	clksrc		: 1;
	unsigned int	hsclksel	: 1;
	unsigned int				: 6;
};

struct osc_ctlBitF {
	unsigned int	osc3en	: 1;
	unsigned int	osc1en	: 1;
	unsigned int	ioscen	: 1;
	unsigned int			: 1;
	unsigned int	osc3wt	: 2;
	unsigned int	ioscwt	: 2;
};

struct osc_nfenBitF {
	unsigned int	nmife	: 1;
	unsigned int	rstfe	: 1;
	unsigned int			: 6;
};

struct osc_lclkBitF {
	unsigned int	lcken	: 1;
	unsigned int	lcksrc	: 1;
	unsigned int	lckdv	: 3;
	unsigned int			: 3;
};

struct osc_foutBitF {
	unsigned int	fout1e	: 1;
	unsigned int	fouthe	: 1;
	unsigned int	fouthd	: 2;
	unsigned int			: 4;
};

struct osc_t8osc1BitF {
	unsigned int	t8o1ce	: 1;
	unsigned int	t8o1ck	: 3;
	unsigned int			: 4;
};

struct osc_svdBitF {
	unsigned int	svdcken	: 1;
	unsigned int	svdsrc	: 1;
	unsigned int			: 6;
};

struct osc_rfcBitF {
	unsigned int	rftcken		: 1;
	unsigned int	rftcksrc	: 1;
	unsigned int	rftckdv		: 2;
	unsigned int				: 4;
};

#if defined(S1C17624) || defined(S1C17604)
struct t16a_clkBitF {
	unsigned int	clken		: 1;
	unsigned int	multimd		: 1;
	unsigned int 	clksrc		: 2;
	unsigned int	clkdiv		: 4;
};

struct rtc_ccBitF {
	unsigned int	rtcce		: 1;
	unsigned int				: 7;
};
#endif // end of S1C17624/S1C17604
#endif	// end of BIT_FIELD


/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define OSC_BASE_ADDR						(BASE_ADDR+OSC_OFFSET)


/***********************************************
 ****** The Definition of Oscillator ***********
 ****** [OSC]                        ***********
 ***********************************************/
// Clock Souce Slect Register
#define R08_OSC_SRC							(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x00)
	#define HSCLKSEL_SFT						1
	#define R08_OSC_SRC_CLKSRC_SFT				0
	#define HSCLKSEL							BIT(0x01, HSCLKSEL_SFT)		// high-speed clock select
		#define SRC_IOSC							0x00						// IOSC
		#define SRC_OSC3							0x01						// OSC3
	#define R08_OSC_SRC_CLKSRC								BIT(0x01, R08_OSC_SRC_CLKSRC_SFT)		// system clock source select
		#define SRC_HSCLK							0x00						// HSCLK
		#define SRC_OSC1							0x01						// OSC1

#ifdef	BIT_FIELD
#define OSC_SRC								*(volatile unsigned char *)(BASE_ADDR+0x1060)
#define BF_OSC_SRC							(*(volatile struct osc_srcBitF *)(BASE_ADDR+0x1060))
	#define OSC_SRC_HSCLKSEL					BF_OSC_SRC.hsclksel			// High-speed clock select
	#define OSC_SRC_CLKSRC						BF_OSC_SRC.clksrc			// System clock source select
#endif	// end of BIT_FIELD


// Oscillation Controle Register
#define R08_OSC_CTL							(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x01)
	#define IOSCWT_SFT							6
	#define OSC3WT_SFT							4
	#define IOSCWT								BIT(0x03, IOSCWT_SFT)		// IOSC wait cycle select
	#define OSC3WT								BIT(0x03, OSC3WT_SFT)		// OSC3 wait cycle select
	#define IOSCEN								BIT(0x01, 2)				// IOSC enable
	#define OSC1EN								BIT(0x01, 1)				// OSC1 enable
	#define OSC3EN								BIT(0x01, 0)				// OSC3 enable
		#define OSC_DISABLE						0x00						// disable
		#define OSC_ENABLE							0x01						// enable
	#define OSCEN_MSK							(IOSCEN | OSC1EN | OSC3EN)

#ifdef	BIT_FIELD
#define OSC_CTL								*(volatile unsigned char *)(BASE_ADDR+0x1061)
#define BF_OSC_CTL							(*(volatile struct osc_ctlBitF *)(BASE_ADDR+0x1061))
	#define OSC_CTL_IOSCWT						BF_OSC_CTL.ioscwt			// IOSC wait cycle select
	#define OSC_CTL_OSC3WT						BF_OSC_CTL.osc3wt			// OSC3 wait cycle select
	#define OSC_CTL_IOSCEN						BF_OSC_CTL.ioscen			// IOSC enable
	#define OSC_CTL_OSC1EN						BF_OSC_CTL.osc1en			// OSC1 enable
	#define OSC_CTL_OSC3EN						BF_OSC_CTL.osc3en			// OSE3 enable
#endif	// end of BIT_FIELD


// Noise Filter Enable Register
#define R08_OSC_NFEN						(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x02)
	#define	RSTFE_SFT							1
	#define	NMIFE_SFT							0
	#define RSTFE								BIT(0x01, RSTFE_SFT)		// Reset noise filter enalbe
	#define NMIFE								BIT(0x01, NMIFE_SFT)		// NMI noise filter enable
	#define NFEN_MSK							(RSTFE | NMIFE)

#ifdef	BIT_FIELD
#define OSC_NFEN							*(volatile unsigned char *)(BASE_ADDR+0x1062)
#define BF_OSC_NFEN							(*(volatile struct osc_nfenBitF *)(BASE_ADDR+0x1062))
	#define OSC_NFEN_RSTFE						BF_OSC_NFEN.rstfe			// Reset noise filter enalbe
	#define OSC_NFEN_NMIFE						BF_OSC_NFEN.nmife			// NMI noise filter enable
#endif	// end of BIT_FIELD


// LCD Clock Setup Register
#define R08_OSC_LCLK						(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x03)
	#define	LCKDV_SFT							2
	#define	LCKSRC_SFT							1
	#define	LCDEN_SFT							0
	#define LCKDV								BIT(0x07, LCKDV_SFT)		// LCD clock division ratio select
	#define LCKSRC								BIT(0x01, LCKSRC_SFT)		// LCD clock source select
		#define LCKSRC_HSCLK						0x00						// HSCLK
		#define LCKSRC_OSC1							0x01						// OSC1
	#define LCKEN								BIT(0x01, LCDEN_SFT)		// LCD clock enable

#ifdef	BIT_FIELD
#define OSC_LCLK							*(volatile unsigned char *)(BASE_ADDR+0x1063)
#define BF_OSC_LCLK							(*(volatile struct osc_lclkBitF *)(BASE_ADDR+0x1063))
	#define OSC_LCLK_LCKDV						BF_OSC_LCLK.lckdv			// LCD clock division ratio select
	#define OSC_LCLK_LCKSRC						BF_OSC_LCLK.lcksrc			// LCD clock source select
	#define OSC_LCLK_LCKEN						BF_OSC_LCLK.lcken			// LCD clock enable
#endif	// end of BIT_FIELD


// FOUT Control Register
#define R08_OSC_FOUT						(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x04)
	#define	FOUTHD_SFT							2
	#define	FOUTHE_SFT							1
	#define	FOUT1E_SFT							0
	#define FOUTHD								BIT(0x03, FOUTHD_SFT)		// FOUTH clock division ratio select
	#define FOUTHE								BIT(0x01, FOUTHE_SFT)		// FOUTH output enable
	#define FOUT1E								BIT(0x01, FOUT1E_SFT)		// FOUT1 output enable
	#define FOUT_MSK							(FOUTHE | FOUT1E)

#ifdef	BIT_FIELD
#define OSC_FOUT							*(volatile unsigned char *)(BASE_ADDR+0x1064)
#define BF_OSC_FOUT							(*(volatile struct osc_foutBitF *)(BASE_ADDR+0x1064))
	#define OSC_FOUT_FOUTHD						BF_OSC_FOUT.fouthd			// FOUTH clock division ratio select
	#define OSC_FOUT_FOUTHE						BF_OSC_FOUT.fouthe			// FOUTH output enable
	#define OSC_FOUT_FOUT1E						BF_OSC_FOUT.fout1e			// FOUT1 output enable
#endif	// end of BIT_FIELD


// T8OSC1 Clock Control Register
#define R08_OSC_T8OSC1						(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x05)
	#define	T8O1CK_SFT							1
	#define	T8O1CE_SFT							0
	#define T8O1CK								BIT(0x07, T8O1CK_SFT)		// T8OSC1 clock division ratio select
	#define T8O1CE								BIT(0x01, T8O1CE_SFT)		// T8OSC1 clock output enable

#ifdef	BIT_FIELD
#define OSC_T8OSC1							*(volatile unsigned char *)(BASE_ADDR+0x1065)
#define BF_OSC_T8OSC1						(*(volatile struct osc_t8osc1BitF *)(BASE_ADDR+0x1065))
	#define OSC_T8OSC1_T8O1CK					BF_OSC_T8OSC1.t8o1ck		// T8OSC1 clock division ratio select
	#define OSC_T8OSC1_T8O1CE					BF_OSC_T8OSC1.t8o1ce		// T8OSC1 clock output enable
#endif	// end of BIT_FIELD


// SVD Clock Control Register
#define R08_OSC_SVD							(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x06)
	#define	SVDSRC_SFT							1
	#define	SVDCKEN_SFT							0
	#define SVDSRC								BIT(0x01, SVDSRC_SFT)		// SVD clock source select
		#define SVDSRC_HSCLK						0x00						// HSCLK1/512
		#define SVDSRC_OSC1							0x01						// OSC1
	#define SVDCKEN								BIT(0x01, SVDCKEN_SFT)		// SVD clock enable

#ifdef	BIT_FIELD
#define OSC_SVD								*(volatile unsigned char *)(BASE_ADDR+0x1066)
#define BF_OSC_SVD							(*(volatile struct osc_svdBitF *)(BASE_ADDR+0x1066))
	#define OSC_SVD_SVDSRC						BF_OSC_SVD.svdsrc			// SVD clock source select
	#define OSC_SVD_SVDCKEN						BF_OSC_SVD.svdcken			// SVD clock enable
#endif	// end of BIT_FIELD


// RFC Clock Control Register
#define R08_OSC_RFC							(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x07)
	#define	RFTCKDV_SFT							2
	#define	RFTCKSRC_SFT						1
	#define	RFTCKEN_SFT							0
	#define RFTCKDV								BIT(0x03, RFTCKDV_SFT)		// RFC clock division ratio select
	#define RFTCKSRC							BIT(0x01, RFTCKSRC_SFT)		// RFC clock source select
		#define RFTCKSRC_HSCLK						0x00						// HSCLK
		#define RFTCKSRC_OSC1						0x01						// OSC1
	#define RFTCKEN								BIT(0x01, RFTCKEN_SFT)		// RFC clock enable

#ifdef	BIT_FIELD
#define OSC_RFC								*(volatile unsigned char *)(BASE_ADDR+0x1067)
#define BF_OSC_RFC							(*(volatile struct osc_rfcBitF *)(BASE_ADDR+0x1067))
	#define OSC_RFC_RFCKDV						BF_OSC_RFC.rftckdv			// RFC clock division ratio select
	#define OSC_RFC_RFCKSRC						BF_OSC_RFC.rftcksrc			// RFC clock source select
	#define OSC_RFC_RFCKEN						BF_OSC_RFC.rftcken			// RFC clock enable
#endif	// end of BIT_FIELD

#if defined(S1C17624) || defined(S1C17604)
// T16A2 Clock Control Register
#define R08_T16A_CLK(ch)						(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x08+ch)
	#define	CLKDIV_SFT							4
	#define	R08_T16A_CLK_CLKSRC_SFT				2
	#define	MULTIMD_SFT							1
	#define	CLKEN_SFT							0
	#define	CLKDIV								BIT(0x0f, CLKDIV_SFT)		// T16A2 clock division ratio select
	#define	R08_T16A_CLK_CLKSRC					BIT(0x03, R08_T16A_CLK_CLKSRC_SFT)		// T16A2 clock source select
	#define	MULTIMD								BIT(0x01, MULTIMD_SFT)		// Multi-Comparator/Capture Mode Select(T16A_CLK0 register)
		#define	T16A_NORMAL							0x00						// normal
		#define	T16A_MULTI							0x01						// multi
	#define	CLKEN								BIT(0x01, CLKEN_SFT)		// T16A2 clock enable
		#define	T16A_CLKDIS							0x00						// disable
		#define	T16A_CLKENA							0x01						// enable

#ifdef	BIT_FIELD
#define T16ACH0_CLK							*(volatile unsigned char *)(BASE_ADDR+0x1068)
#define BF_T16ACH0_CLK						(*(volatile struct t16a_clkBitF *)(BASE_ADDR+0x1068))
	#define	T16ACH0_CLK_CLKDIV				BF_T16ACH0_CLK.clkdiv			// T16A2 CH0 clock division ratio select
	#define	T16ACH0_CLK_CLKSRC				BF_T16ACH0_CLK.clksrc			// T16A2 CH0 clock source select
	#define	T16ACH0_CLK_MULTIMD				BF_T16ACH0_CLK.multimd			// Multi-Comparator/Capture
	#define	T16ACH0_CLK_CLKEN				BF_T16ACH0_CLK.clken			// T16A2 CH0 clock enable

#define T16ACH1_CLK							*(volatile unsigned char *)(BASE_ADDR+0x1069)
#define BF_T16ACH1_CLK						(*(volatile struct t16a_clkBitF *)(BASE_ADDR+0x1069))
	#define	T16ACH1_CLK_CLKDIV				BF_T16ACH1_CLK.clkdiv			// T16A2 CH1 clock division ratio select
	#define	T16ACH1_CLK_CLKSRC				BF_T16ACH1_CLK.clksrc			// T16A2 CH1 clock source select
	#define	T16ACH1_CLK_CLKEN				BF_T16ACH1_CLK.clken			// T16A2 CH1 clock enable
#endif // end of BIT_FIELD

// RTC Clock Control Register
#define R08_RTC_CC								(volatile unsigned char *)(unsigned long)(OSC_BASE_ADDR+0x0e)
	#define	RTCCE_SFT							0
	#define	RTCCE								BIT(0x01, RTCCE_SFT)		// RTC clock enable
		#define	RTC_CLKDIS							0x00						// disable
		#define	RTC_CLKENA							0x01						// enable

#ifdef	BIT_FIELD
#define RTC_CC									*(volatile unsigned char *)(BASE_ADDR+0x106e)
#define BF_RTC_CC							(*(volatile struct rtc_ccBitF *)(BASE_ADDR+0x106e))
	#define	RTC_CC_RTCCE					BF_RTC_CC.rtcce					// RTC clock enable
#endif // end of BIT_FIELD
#endif // end of S1C17624/S1C17604

#endif	// end of OSC_H_
