/**
 * @file	psc.c
 * @brief	PSC driver
 * 
 * - Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 */

#include <s1c176xx.h>

#ifdef	PE_PSC
#include	<psc_api.h>

/*
 * definition of "define"
 */


/*
 * definition of constants
 */


/*
 * Declarations of function prototype
 */
short PSC_setPrescalerControl(unsigned char modeDebug, unsigned char modeNormal);


/**
 * @brief	set a prescaler run / stop.
 * - 1. check parameters.
 * - 2. set a prescaler run/stop.
 *
 * @param	modeDebug; unsigned char: 		prescaler for debug mode run / stop
 * -		PSC_DEBUG_STOP : prescaler stop for debug mode
 * -		PSC_DEBUG_RUN  : prescaler run for debug mode
 * @param	modeNormal; unsigned char: 		prescaler control run / stop
 * -		PSC_NORMAL_STOP : prescaler stop
 * -		PSC_NORMAL_RUN  : prescaler run
 * @return	short:							end status
 * -		PSC_SUCCESS : success
 * -		PSC_INVALID_PARAMETER : invalid parameter
 * @date	2009/12/25
 */
short PSC_setPrescalerControl(unsigned char modeDebug, unsigned char modeNormal)
{
	// parameter check
	if(	(modeDebug > PSC_DEBUG_RUN) ||
		(modeNormal > PSC_NORMAL_RUN) ) {
			return	PSC_INVALID_PARAMETER;
	}
	
	// prescaler run / stop set
	RegWrite(R08_PSC_CTL, ((modeDebug << PRUND_SFT) | (modeNormal << PSC_PRUN_SFT)));
	
	return	PSC_SUCCESS;
}

#endif	// end of PE_PSC
