/******************************************************************
 * rtc.h
 *
 * real time clock driver
 *
 * Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 *****************************************************************/

#ifndef RTC_H_
#define RTC_H_

#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct rtc_instatBitF {
	unsigned int	rtcirq	: 1;
	unsigned int			: 7;
};

struct rtc_intmodeBitF {
	unsigned int	rtcien	: 1;
	unsigned int	rtcimd	: 1;
	unsigned int	rtct	: 3;
	unsigned int			: 3;
};

struct rtc_cntl0BitF {
	unsigned int	rtcrst	: 1;
	unsigned int	rtcstp	: 1;
	unsigned int	rtcadj	: 1;
	unsigned int			: 1;
	unsigned int	rtc24h	: 1;
	unsigned int			: 3;
};

struct rtc_cntl1BitF {
	unsigned int	rtchld	: 1;
	unsigned int	rtcbsy	: 1;
	unsigned int	rtcrdhld: 1;
	unsigned int			: 5;
};

struct rtc_secBitF {
	unsigned int	rtcsl	: 4;
	unsigned int	rtcsh	: 3;
	unsigned int			: 1;
};

struct rtc_minBitF {
	unsigned int	rtcmil	: 4;
	unsigned int	rtcmih	: 3;
	unsigned int			: 1;
};

struct rtc_hourBitF {
	unsigned int	rtchl	: 4;
	unsigned int	rtchh	: 2;
	unsigned int	rtcap	: 1;
	unsigned int			: 1;
};

struct rtc_dayBitF {
	unsigned int	rtcdl	: 4;
	unsigned int	rtcdh	: 2;
	unsigned int			: 2;
};

struct rtc_monthBitF {
	unsigned int	rtcmol	: 4;
	unsigned int	rtcmoh	: 1;
	unsigned int			: 3;
};

struct rtc_yearBitF {
	unsigned int	rtcyl	: 4;
	unsigned int	rtcyh	: 4;
};

struct rtc_weekBitF {
	unsigned int	rtcwk	: 3;
	unsigned int			: 5;
};
#endif	// end of BIT_FIELD

/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define	RTC_BASE_ADDR						(BASE_ADDR+RTC_OFFSET)


/***********************************************
 ****** The Definition of Real Time Clock  *****
 ****** [RTC]                              *****
 ***********************************************/
// RTC Interrupt Status Register
#define	R08_RTC_INSTAT							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x00)
	#define	RTCIRQ_SFT								0
	#define	RTCIRQ									BIT(0x01, RTCIRQ_SFT)		// Interrupt status
		#define	RTC_INVALID								0x00						// Invalid
		#define	RTC_RESET								0x01						// Reset
		#define	RTC_NOTOCCURRED							0x00						// Not occurred
		#define	RTC_OCCURRED								0x01						// Occurred

#ifdef	BIT_FIELD
#define RTC_INSTAT								*(volatile unsigned char *)(BASE_ADDR+0x1140)
#define BF_RTC_INSTAT							(*(volatile struct rtc_instatBitF *)(BASE_ADDR+0x1140))
	#define RTC_INSTAT_RTCIRQ					BF_RTC_INSTAT.rtcirq				// Interrupt status
#endif	// end of BIT_FIELD

// RTC Interrupt Mode Register
#define	R08_RTC_INTMODE							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x01)
	#define	RTCT_SFT								2
	#define	RTCIMD_SFT								1
	#define	RTCIEN_SFT								0
	#define	RTCT									BIT(0x07, RTCT_SFT) 		// RTC interrupt cycle setup
		#define	RTC_SECOND_64							0x00						// 1/64 second
		#define	RTC_SECOND								0x01						// 1 second
		#define	RTC_MINUTE								0x02						// 1 minute
		#define	RTC_HOUR_1								0x03						// 1 hour
		#define	RTC_SECOND_512							0x04						// 1/512 second
		#define	RTC_SECOND_256							0x05						// 1/256 second
		#define	RTC_SECOND_128							0x06						// 1/128 second
	#define	RTCIMD									BIT(0x01, RTCIMD_SFT)		// RTC interrupt mode select
		#define	RTC_EDGE								0x00						// Edge trigger
		#define	RTC_LEVEL								0x01						// Level sense
	#define	RTCIEN									BIT(0x01, RTCIEN_SFT)		// RTC interrupt enable
		#define	RTC_DISABLE								0x00						// Disable
		#define	RTC_ENABLE								0x01						// Enable

#ifdef BIT_FIELD
#define RTC_INTMODE							*(volatile unsigned char *)(BASE_ADDR+0x1141)
#define BF_RTC_INTMODE							(*(volatile struct rtc_intmodeBitF *)(BASE_ADDR+0x1141))
	#define RTC_INTMODE_RTCT						BF_RTC_INTMODE.rtct					// RTC interrupt cycle setup
	#define RTC_INTMODE_RTCIMD						BF_RTC_INTMODE.rtcimd				// RTC interrupt mode select
	#define RTC_INTMODE_RTCIEN						BF_RTC_INTMODE.rtcien				// RTC interrupt enable
#endif // end of BIT_FIELD

// RTC Control 0 Register
#define	R08_RTC_CNTL0							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x02)
	#define	RTC24H_SFT								4
	#define	RTCADJ_SFT								2
	#define	RTCSTP_SFT								1
	#define	RTCRST_SFT								0
	#define	RTC24H									BIT(0x01, RTC24H_SFT)		// 24H/12H mode select
		#define	RTC_12H									0x00						// 12H
		#define	RTC_24H									0x01						// 24H
	#define	RTCADJ									BIT(0x01, RTCADJ_SFT)		// 30-second adjustment
		#define	RTC_NORMAL								0x00						// Normal
		#define	RTC_ADJUST								0x01						// Adjust
	#define	RTCSTP									BIT(0x01, RTCSTP_SFT)		// Divider run/stop control
		#define	RTC_RUN									0x00						// Run
		#define	RTC_STOP								0x01						// Stop
	#define	RTCRST									BIT(0x01, RTCRST_SFT)		// Software reset
		#define	RTC_RSTCANCEL							0x00						// Ignored
		#define	RTC_SWRESET								0x01						// Software reset

#ifdef BIT_FIELD
#define RTC_CNTL0								*(volatile unsigned char *)(BASE_ADDR+0x1142)
#define BF_RTC_CNTL0							(*(volatile struct rtc_cntl0BitF *)(BASE_ADDR+0x1142))
	#define RTC_CNTL0_RTC24H						BF_RTC_CNTL0.rtc24h					// 24H/12H mode select
	#define RTC_CNTL0_RTCADJ						BF_RTC_CNTL0.rtcadj					// 30-second adjustment
	#define RTC_CNTL0_RTCSTP						BF_RTC_CNTL0.rtcstp					// Divider run/stop control
	#define RTC_CNTL0_RTCRST						BF_RTC_CNTL0.rtcrst					// Software reset
#endif // end of BIT_FIELD

// RTC CONTROL 1 Register
#define	R08_RTC_CNTL1							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x03)
	#define	RTCRDHLD_SFT							2
	#define	RTCBSY_SFT								1
	#define	RTCHLD_SFT								0
	#define	RTCRDHLD								BIT(0x01, RTCRDHLD_SFT)		// Read buffer enable
		#define	RTC_RBOFF								0x00						// Disable
		#define	RTC_RBON								0x01						// Enable
	#define	RTCBSY									BIT(0x01, RTCBSY_SFT)		// Counter busy flag
		#define	RTC_POSSIBLE							0x00						// R/W possible
		#define	RTC_BUSY								0x01						// Busy
	#define	RTCHLD									BIT(0x01, RTCHLD_SFT)		// Counter busy flag
		#define	RTC_RUNNING								0x00						// Running
		#define	RTC_HOLD								0x01						// Hold

#ifdef BIT_FIELD
#define RTC_CNTL1								*(volatile unsigned char *)(BASE_ADDR+0x1143)
#define BF_RTC_CNTL1							(*(volatile struct rtc_cntl1BitF *)(BASE_ADDR+0x1143))
	#define RTC_CNTL1_RTCRDHLD						BF_RTC_CNTL1.rtcrdhld				// Read buffer enable
	#define RTC_CNTL1_RTCBSY						BF_RTC_CNTL1.rtcbsy					// Counter busy flag
	#define RTC_CNTL1_RTCHLD						BF_RTC_CNTL1.rtchld					// Counter hold control
#endif // end of BIT_FIELD

// RTC counter value
#define	RTC_MAX									9							// RTC counter max

// RTC Second Register
#define	R08_RTC_SEC								(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x04)
	#define	RTCSH_SFT								4
	#define	RTCSL_SFT								0
	#define	RTCSH									BIT(0x07, RTCSH_SFT)		// RTC 10-second counter
	#define	RTCSL									BIT(0x0f, RTCSL_SFT)		// RTC 1-second counter
		#define	RTC_SMAX								0x59						// RTC second counter max

#ifdef BIT_FIELD
#define RTC_SEC								*(volatile unsigned char *)(BASE_ADDR+0x1144)
#define BF_RTC_SEC								(*(volatile struct rtc_secBitF *)(BASE_ADDR+0x1144))
	#define RTC_SEC_RTCSH							BF_RTC_SEC.rtcsh					// RTC 10-second counter
	#define RTC_SEC_RTCSL							BF_RTC_SEC.rtcsl					// RTC 1-second counter
#endif // end of BIT_FIELD

// RTC Minute Register
#define	R08_RTC_MIN								(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x05)
	#define	RTCMIH_SFT								4
	#define	RTCMIL_SFT								0
	#define	RTCMIH									BIT(0x07, RTCMIH_SFT)		// RTC 10-minute counter
	#define	RTCMIL									BIT(0x0f, RTCMIL_SFT)		// RTC 1-minute counter
		#define	RTC_MIMAX								0x59						// RTC minute counter max

#ifdef BIT_FIELD
#define RTC_MIN								*(volatile unsigned char *)(BASE_ADDR+0x1145)
#define BF_RTC_MIN								(*(volatile struct rtc_minBitF *)(BASE_ADDR+0x1145))
	#define RTC_MIN_RTCMIH							BF_RTC_MIN.rtcmih					// RTC 10-minute counter
	#define RTC_MIN_RTCMIL							BF_RTC_MIN.rtcmil					// RTC 1-minute counter
#endif // end of BIT_FIELD

// RTC Hour Register
#define	R08_RTC_HOUR							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x06)
	#define	RTCAP_SFT								6
	#define	RTCHH_SFT								4
	#define	RTCHL_SFT								0
	#define	RTCAP									BIT(0x01, RTCAP_SFT)		// AM/PM indicator
		#define	RTC_AM									0x00					// AM
		#define	RTC_PM									0x01					// PM
	#define	RTCHH									BIT(0x03, RTCHH_SFT)		// RTC 10-hour counter
	#define	RTCHL									BIT(0x0f, RTCHL_SFT)		// RTC 1-hour counter
		#define	RTC_HMAX								0x23						// RTC hour counter max

#ifdef BIT_FIELD
#define RTC_HOUR								*(volatile unsigned char *)(BASE_ADDR+0x1146)
#define BF_RTC_HOUR							(*(volatile struct rtc_hourBitF *)(BASE_ADDR+0x1146))
	#define RTC_HOUR_RTCAP							BF_RTC_HOUR.rtcap					// AM/PM indicator
	#define RTC_HOUR_RTCHH							BF_RTC_HOUR.rtchh					// RTC 10-hour counter
	#define RTC_HOUR_RTCHL							BF_RTC_HOUR.rtchl					// RTC 1-hour counter
#endif // end of BIT_FIELD

// RTC Day Register
#define	R08_RTC_DAY								(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x07)
	#define	RTCDH_SFT								4
	#define	RTCDL_SFT								0
	#define	RTCDH									BIT(0x03, RTCDH_SFT)		// RTC 10-day counter
	#define	RTCDL									BIT(0x0f, RTCDL_SFT)		// RTC 1-day counter
		#define	RTC_DMIN								0x01						// RTC day counter min
		#define	RTC_DMAX								0x31						// RTC day counter max

#ifdef BIT_FIELD
#define RTC_DAY								*(volatile unsigned char *)(BASE_ADDR+0x1147)
#define BF_RTC_DAY								(*(volatile struct rtc_dayBitF *)(BASE_ADDR+0x1147))
	#define RTC_DAY_RTCDH							BF_RTC_DAY.rtcdh					// RTC 10-day counter
	#define RTC_DAY_RTCDL							BF_RTC_DAY.rtcdl					// RTC 1-day counter
#endif // end of BIT_FIELD

// RTC Month Register
#define	R08_RTC_MONTH							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x08)
	#define	RTCMOH_SFT								4
	#define	RTCMOL_SFT								0
	#define	RTCMOH									BIT(0x01, RTCMOH_SFT)		// RTC 10-month counter
	#define	RTCMOL									BIT(0x0f, RTCMOL_SFT)		// RTC 1-month counter
		#define	RTC_MOMIN								0x01						// RTC month counter min
		#define	RTC_MOMAX								0x12						// RTC month counter max

#ifdef BIT_FIELD
#define RTC_MONTH								*(volatile unsigned char *)(BASE_ADDR+0x1148)
#define BF_RTC_MONTH							(*(volatile struct rtc_monthBitF *)(BASE_ADDR+0x1148))
	#define RTC_MONTH_RTCMOH						BF_RTC_MONTH.rtcmoh					// RTC 10-month counter
	#define RTC_MONTH_RTCMOL						BF_RTC_MONTH.rtcmol					// RTC 1-month counter
#endif // end of BIT_FIELD

// RTC Year Register
#define	R08_RTC_YEAR							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x09)
	#define	RTCYH_SFT								4
	#define	RTCYL_SFT								0
	#define	RTCYH									BIT(0x0f, RTCYH_SFT)		// RTC 10-year counter
	#define	RTCYL									BIT(0x0f, RTCYL_SFT)		// RTC 1-year counter
		#define	RTC_YMAX								0x99						// RTC year counter max

#ifdef BIT_FIELD
#define RTC_YEAR								*(volatile unsigned char *)(BASE_ADDR+0x1149)
#define BF_RTC_YEAR							(*(volatile struct rtc_yearBitF *)(BASE_ADDR+0x1149))
	#define RTC_YEAR_RTCYH							BF_RTC_YEAR.rtcyh					// RTC 10-year counter
	#define RTC_YEAR_RTCYL							BF_RTC_YEAR.rtcyl					// RTC 1-year counter
#endif // end of BIT_FIELD

// RTC Days of Week Register
#define	R08_RTC_WEEK							(volatile unsigned char *)(unsigned long)(RTC_BASE_ADDR+0x0a)
	#define	RTCWK_SFT								0
	#define	RTCWK									BIT(0x07, RTCWK_SFT)		// RTC days of week counter

#ifdef BIT_FIELD
#define RTC_WEEK								*(volatile unsigned char *)(BASE_ADDR+0x114a)
#define BF_RTC_WEEK							(*(volatile struct rtc_weekBitF *)(BASE_ADDR+0x114a))
	#define RTC_WEEK_RTCWK							BF_RTC_WEEK.rtcwk					// RTC days of week counter
#endif // end of BIT_FIELD


#endif	// end of RTC_H_
