/******************************************************************
 * spi.h
 * 
 * SPI driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef SPI_H_
#define SPI_H_


#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct spi_stBitF {
	unsigned int	sptbe	: 1;
	unsigned int	sprbf	: 1;
	unsigned int	spbsy	: 1;
	unsigned int			: 13;
};

struct spi_txdBitF {
	unsigned int	sptdb	: 8;
	unsigned int			: 8;
};

struct spi_rxdBitF {
	unsigned int	sprdb	: 8;
	unsigned int			: 8;
};

struct spi_ctlBitF {
	unsigned int	spen	: 1;
	unsigned int	mssl	: 1;
	unsigned int	cpol	: 1;
	unsigned int	cpha	: 1;
	unsigned int	sptie	: 1;
	unsigned int	sprie	: 1;
	unsigned int			: 2;
	unsigned int	mlsb	: 1;
	unsigned int	mclk	: 1;
	unsigned int			: 6;
};
#endif	// end of BIT_FIELD


/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define	SPI_BASE_ADDR(ch)					(BASE_ADDR+(ch ? SPI_OFFSET2 : SPI_OFFSET))


/***********************************************
 ****** The Definition of SPI ******************
 ****** [SPI]                 ******************
 ***********************************************/
// SPI Status Register
#define	R16_SPI_ST(ch)						(volatile unsigned short *)(unsigned long)(SPI_BASE_ADDR(ch)+0x00+(ch*SPI_CHOFFSET))
	#define	SPBSY_SFT							2
	#define	SPRBF_SFT							1
	#define	SPTBE_SFT							0
	#define	SPBSY								BIT(0x0001, SPBSY_SFT)		// Transfer busy flag(master) / ss signal low flag(slave)
		#define	SPI_IDLE							0x0000						// idle(master)
		#define	SPI_BUSY							0x0001						// busy(master)
		#define	SPI_SSH								0x0000						// ss = high(slave)
		#define	SPI_SSL								0x0001						// ss = low(slave)
	#define	SPRBF								BIT(0x0001, SPRBF_SFT)		// Receive data buffer full flag
		#define	SPI_NOTFULL							0x0000						// not full
		#define	SPI_FULL							0x0001						// full
	#define	SPTBE								BIT(0x0001, SPTBE_SFT)		// Transmit data buffer empty flag
		#define	SPI_NOTEMPTY						0x0000						// not empty
		#define	SPI_EMPTY							0x0001						// empty

#ifdef	BIT_FIELD
#define SPI_ST								*(volatile unsigned short *)(BASE_ADDR+0x0320)
#define BF_SPI_ST							(*(volatile struct spi_stBitF *)(BASE_ADDR+0x0320))
	#define SPI_ST_SPBSY						BF_SPI_ST.spbsy				// Transfer busy flag(master) / ss signal low flag(slave)
	#define SPI_ST_SPRBF						BF_SPI_ST.sprbf				// Receive data buffer full flag
	#define SPI_ST_SPTBE						BF_SPI_ST.sptbe				// Transmit data buffer empty flag
#endif	// end of BIT_FIELD


// SPI Transmit Data Register
#define	R16_SPI_TXD(ch)						(volatile unsigned short *)(unsigned long)(SPI_BASE_ADDR(ch)+0x02+(ch*SPI_CHOFFSET))
	#define	SPTDB_SFT							0
	#define	SPTDB								BIT(0x00ff, SPTDB_SFT)		// SPI transmit data buffer

#ifdef	BIT_FIELD
#define SPI_TXD								*(volatile unsigned short *)(BASE_ADDR+0x0322)
#define BF_SPI_TXD							(*(volatile struct spi_txdBitF *)(BASE_ADDR+0x0322))
	#define SPI_TXD_SPTDB						BF_SPI_TXD.sptdb			// SPI transmit data buffer
#endif	// end of BIT_FIELD


// SPI Receive Data Register
#define	R16_SPI_RXD(ch)						(volatile unsigned short *)(unsigned long)(SPI_BASE_ADDR(ch)+0x04+(ch*SPI_CHOFFSET))
	#define	SPRDB_SFT							0
	#define	SPRDB								BIT(0x00ff, SPRDB_SFT)		// SPI receive data buffer

#ifdef	BIT_FIELD
#define SPI_RXD								*(volatile unsigned short *)(BASE_ADDR+0x0324)
#define BF_SPI_RXD							(*(volatile struct spi_rxdBitF *)(BASE_ADDR+0x0324))
	#define SPI_RXD_SPRDB						BF_SPI_RXD.sprdb			// SPI receive data buffer
#endif	// end of BIT_FIELD


// SPI Control Register
#define	R16_SPI_CTL(ch)						(volatile unsigned short *)(unsigned long)(SPI_BASE_ADDR(ch)+0x06+(ch*SPI_CHOFFSET))
	#define	MCLK_SFT							9
	#define	MLSB_SFT							8
	#define	SPRIE_SFT							5
	#define	SPTIE_SFT							4
	#define	CPHA_SFT							3
	#define	CPOL_SFT							2
	#define	MSSL_SFT							1
	#define	SPEN_SFT							0
	#define	MCLK								BIT(0x0001, MCLK_SFT)		// SPI clock source select
		#define	SPI_PCLK							0x0000					// PCLK1/4
		#define	SPI_T16								0x0001					// T16 Ch.1
	#define	MLSB								BIT(0x0001, MLSB_SFT)		// LSB/MSB first mode select
		#define	SPI_MSB								0x0000					// MSB first
		#define	SPI_LSB								0x0001					// LSB first
	#define	SPRIE								BIT(0x0001, SPRIE_SFT)		// Receive data buffer full interrupt enable
		#define	SPI_DISABLE							0x0000						// disable
		#define	SPI_ENABLE							0x0001						// enable
	#define	SPTIE								BIT(0x0001, SPTIE_SFT)		// Transmit data buffer empty interrupt enable
		#define	SPI_IEN_MASK						(SPRIE | SPTIE)				// interrupt mask data
	#define	CPHA								BIT(0x0001, CPHA_SFT)		// SPI Clock phase select
		#define	SPI_DATA_IN							0x0000						// data in
		#define	SPI_DATA_OUT						0x0001						// data out
	#define	CPOL								BIT(0x0001, CPOL_SFT)		// SPI Clock polarity select
		#define	SPI_ACTH							0x0000						// active high
		#define	SPI_ACTL							0x0001						// active low
	#define	MSSL								BIT(0x0001, MSSL_SFT)		// Master/slave mode select
		#define	SPI_SLAVE							0x0000						// slave mode
		#define	SPI_MASTER							0x0001						// master mode
		#define	SPI_INIT_MASK						(MCLK | MLSB | CPHA | CPOL | MSSL)
																				// initialize mask data
	#define	SPEN								BIT(0x0001, SPEN_SFT)		// SPI enable

#ifdef	BIT_FIELD
#define SPI_CTL								*(volatile unsigned short *)(BASE_ADDR+0x0326)
#define BF_SPI_CTL							(*(volatile struct spi_ctlBitF *)(BASE_ADDR+0x0326))
	#define SPI_CTL_MCLK						BF_SPI_CTL.mclk				// SPI clock source select
	#define SPI_CTL_MLSB						BF_SPI_CTL.mlsb				// LSB/MSB first mode select
	#define SPI_CTL_SPRIE						BF_SPI_CTL.sprie			// Receive data buffer full int. enable
	#define SPI_CTL_SPTIE						BF_SPI_CTL.sptie			// Transmit data buffer empty int. enable
	#define SPI_CTL_CPHA						BF_SPI_CTL.cpha				// Clock phase select
	#define SPI_CTL_CPOL						BF_SPI_CTL.cpol				// Clock polarity select
	#define SPI_CTL_MSSL						BF_SPI_CTL.mssl				// Master/slave mode select
	#define SPI_CTL_SPEN						BF_SPI_CTL.spen				// SPI enable
#endif	// end of BIT_FIELD


#endif	// end of SPI_H_
