/******************************************************************
 * t16a2.h
 *
 * PWM timer(T16A2) driver
 *
 * Copyright(C) SEIKO EPSON CORPORATION 2010. All rights Reserved.
 *****************************************************************/
#ifndef T16A2_H_
#define T16A2_H_


#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct t16a_ctlBitF {
	unsigned int	t16arun	: 1;
	unsigned int	t16arst	: 1;
	unsigned int	t16atrmd: 1;
	unsigned int	cbufen	: 1;
	unsigned int	ccabcnt	: 2;
	unsigned int	hcm		: 1;
	unsigned int			: 9;
};

struct t16a_ccctlBitF {
	unsigned int	ccamd	: 1;
	unsigned int	toutainv: 1;
	unsigned int			: 2;
	unsigned int	toutamd	: 2;
	unsigned int	capatrg	: 2;
	unsigned int	ccbmd	: 1;
	unsigned int	toutbinv: 1;
	unsigned int			: 2;
	unsigned int	toutbmd	: 2;
	unsigned int	capbtrg	: 2;
};

struct t16a_ienBitF {
	unsigned int	caie	: 1;
	unsigned int	cbie	: 1;
	unsigned int	capaie	: 1;
	unsigned int	capbie	: 1;
	unsigned int	capaowie: 1;
	unsigned int	capbowie: 1;
	unsigned int			: 10;
};

struct t16a_iflgBitF {
	unsigned int	caif	: 1;
	unsigned int	cbif	: 1;
	unsigned int	capaif	: 1;
	unsigned int	capbif	: 1;
	unsigned int	capaowif: 1;
	unsigned int	capbowif: 1;
	unsigned int			: 10;
};
#endif	// end of BIT_FIELD


/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define	T16A_BASE_ADDR						(BASE_ADDR+T16A2_OFFSET)


/***********************************************
 ****** The Definition of PWM & Capture Timer **
 ****** [T16A2]                                **
 ***********************************************/
// PWM Timer(T16A2) Control Register
#define	R16_T16A_CTL(ch)					(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x00+(ch*T16A2_CHOFFSET))
	#define	HCM_SFT								6
	#define	CCABCNT_SFT							4
	#define	R16_T16A_CTL_CBUFEN_SFT				3
	#define	T16ATRMD_SFT						2
	#define	T16ARST_SFT							1
	#define	T16ARUN_SFT							0
	#define	HCM									BIT(0x0001, HCM_SFT)			// Half clock mode enable
	#define	CCABCNT								BIT(0x0003, CCABCNT_SFT)		// Counter Select
	#define	R16_T16A_CTL_CBUFEN					BIT(0x0001, R16_T16A_CTL_CBUFEN_SFT)			// Compare Buffer Enable
		#define	T16A_DISABLE						0x0000							// Disable
		#define	T16A_ENABLE							0x0001							// Enable
	#define	T16ATRMD							BIT(0x0001, T16ATRMD_SFT)		// Count Mode Select
		#define	T16A_REPEAT							0x0000							// repeat
		#define	T16A_ONESHOT						0x0001							// one-shot
	#define	T16ARST								BIT(0x0001, T16ARST_SFT)		// Counter Reset
		#define	T16A_INVALID						0x0000							// invalid
		#define	T16A_RESET							0x0001							// reset
	#define	T16ARUN								BIT(0x0001, T16ARUN_SFT)		// Counter Run/Stop Control
		#define	T16A_STOP							0x0000							// stop
		#define	T16A_RUN							0x0001							// run

#ifdef	BIT_FIELD
#define T16ACH0_CTL						*(volatile unsigned short *)(BASE_ADDR+0x1400)
#define BF_T16ACH0_CTL						(*(volatile struct t16a_ctlBitF *)(BASE_ADDR+0x1400))
	#define T16ACH0_CTL_HCM0						BF_T16ACH0_CTL.hcm				// T16A2 CH0 Half clock mode
	#define T16ACH0_CTL_CCABCNT0					BF_T16ACH0_CTL.ccabcnt			// T16A2 CH0 Attached counter select
	#define T16ACH0_CTL_CBUFEN0					BF_T16ACH0_CTL.cbufen			// T16A2 CH0 Compare buffer enable
	#define T16ACH0_CTL_TRMD0						BF_T16ACH0_CTL.t16atrmd			// T16A2 CH0 Counter mode
	#define T16ACH0_CTL_PRESET0					BF_T16ACH0_CTL.t16arst			// T16A2 CH0 Counter reset
	#define T16ACH0_CTL_PRUN0						BF_T16ACH0_CTL.t16arun			// T16A2 CH0 Counter Run/Stop

#define T16ACH1_CTL						*(volatile unsigned short *)(BASE_ADDR+0x1420)
#define BF_T16ACH1_CTL						(*(volatile struct t16a_ctlBitF *)(BASE_ADDR+0x1420))
	#define T16ACH1_CTL_HCM1						BF_T16ACH1_CTL.hcm				// T16A2 CH1 Half clock mode
	#define T16ACH1_CTL_CCABCNT1					BF_T16ACH1_CTL.ccabcnt			// T16A2 CH1 Attached counter select
	#define T16ACH1_CTL_CBUFEN1					BF_T16ACH1_CTL.cbufen			// T16A2 CH1 Compare buffer enable
	#define T16ACH1_CTL_TRMD1						BF_T16ACH1_CTL.t16atrmd			// T16A2 CH1 Counter mode
	#define T16ACH1_CTL_PRESET1					BF_T16ACH1_CTL.t16arst			// T16A2 CH1 Counter reset
	#define T16ACH1_CTL_PRUN1						BF_T16ACH1_CTL.t16arun			// T16A2 CH1 Counter Run/Stop
#endif	// end of BIT_FIELD


// PAM Timer(T16A2) Counter Data Register
#define	R16_T16A_TC(ch)						(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x02+(ch*T16A2_CHOFFSET))

#ifdef	BIT_FIELD
#define T16ACH0_TC							*(volatile unsigned short *)(BASE_ADDR+0x1402)
	#define T16ACH0_TC0							T16ACH0_TC						// T16A2 CH0 Counter data

#define T16ACH1_TC							*(volatile unsigned short *)(BASE_ADDR+0x1422)
	#define T16ACH1_TC1							T16ACH1_TC						// T16A2 CH0 Counter data
#endif	// end of BIT_FIELD


// PWM Timer(T16A2) Comparator/Capture Control Register
#define	R16_T16A_CCCTL(ch)					(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x04+(ch*T16A2_CHOFFSET))
	#define	CAPBTRG_SFT							14
	#define	TOUTBMD_SFT							12
	#define	TOUTBINV_SFT						9
	#define	CCBMD_SFT							8
	#define	CAPATRG_SFT							6
	#define	TOUTAMD_SFT							4
	#define	TOUTAINV_SFT						1
	#define	CCAMD_SFT							0
	#define	CAPBTRG								BIT(0x0003, CAPBTRG_SFT)		// Capture B Trigger Select
	#define	TOUTBMD								BIT(0x0003, TOUTBMD_SFT)		// TOUT B Mode Select
	#define	TOUTBINV							BIT(0x0001, TOUTBINV_SFT)		// TOUT B Invert
		#define	T16A_INVERT							0x0001							// invert
	#define	CCBMD								BIT(0x0001, CCBMD_SFT)			// T16A_CCB Register Mode Select
		#define	T16A_COMPARATOR						0x0000							// comparator
		#define	T16A_CAPTURE						0x0001							// capture
	#define	CAPATRG								BIT(0x0003, CAPATRG_SFT)		// Capture A Trigger Select
	#define	TOUTAMD								BIT(0x0003, TOUTAMD_SFT)		// TOUT A Mode Select
	#define	TOUTAINV							BIT(0x0001, TOUTAINV_SFT)		// TOUT A Invert
	#define	CCAMD								BIT(0x0001, CCAMD_SFT)			// T16A_CCA Register Mode Select

#ifdef	BIT_FIELD
#define T16ACH0_CCCTL						*(volatile unsigned short *)(BASE_ADDR+0x1404)
#define BF_T16ACH0_CCCTL					(*(volatile struct t16a_ccctlBitF *)(BASE_ADDR+0x1404))
	#define T16ACH0_CCCTL_CAPBTRG0					BF_T16ACH0_CCCTL.capbtrg			// T16A2 CH0 Capture B trigger
	#define T16ACH0_CCCTL_TOUTBMD0					BF_T16ACH0_CCCTL.toutbmd			// T16A2 CH0 Timer OUT B mode
	#define T16ACH0_CCCTL_TOUTINVB0				BF_T16ACH0_CCCTL.toutbinv			// T16A2 CH0 Timer OUT B register
	#define T16ACH0_CCCTL_CCBMD0					BF_T16ACH0_CCCTL.ccbmd				// T16A2 CH0 CCB register mode
	#define T16ACH0_CCCTL_CAPATRG0					BF_T16ACH0_CCCTL.capatrg			// T16A2 CH0 Capture A trigger
	#define T16ACH0_CCCTL_TOUTAMD0					BF_T16ACH0_CCCTL.toutamd			// T16A2 CH0 Timer OUT A mode
	#define T16ACH0_CCCTL_TOUTINVA0				BF_T16ACH0_CCCTL.toutainv			// T16A2 CH0 Timer OUT A register
	#define T16ACH0_CCCTL_CCAMD0					BF_T16ACH0_CCCTL.ccamd				// T16A2 CH0 CCA register mode

#define T16ACH1_CCCTL						*(volatile unsigned short *)(BASE_ADDR+0x1424)
#define BF_T16ACH1_CCCTL					(*(volatile struct t16a_ccctlBitF *)(BASE_ADDR+0x1424))
	#define T16ACH1_CCCTL_CAPBTRG1					BF_T16ACH1_CCCTL.capbtrg			// T16A2 CH1 Capture B trigger
	#define T16ACH1_CCCTL_TOUTBMD1					BF_T16ACH1_CCCTL.toutbmd			// T16A2 CH1 Timer OUT B mode
	#define T16ACH1_CCCTL_TOUTINVB1				BF_T16ACH1_CCCTL.toutbinv			// T16A2 CH1 Timer OUT B register
	#define T16ACH1_CCCTL_CCBMD1					BF_T16ACH1_CCCTL.ccbmd				// T16A2 CH1 CCB register mode
	#define T16ACH1_CCCTL_CAPATRG1					BF_T16ACH1_CCCTL.capatrg			// T16A2 CH1 Capture A trigger
	#define T16ACH1_CCCTL_TOUTAMD1					BF_T16ACH1_CCCTL.toutamd			// T16A2 CH1 Timer OUT A mode
	#define T16ACH1_CCCTL_TOUTINVA1				BF_T16ACH1_CCCTL.toutainv			// T16A2 CH1 Timer OUT A register
	#define T16ACH1_CCCTL_CCAMD1					BF_T16ACH1_CCCTL.ccamd				// T16A2 CH1 CCA register mode
#endif	// end of BIT_FIELD


// PWM Timer(T16A2) Comparator/Capture A Data Register
#define	R16_T16A_CCA(ch)					(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x06+(ch*T16A2_CHOFFSET))

#ifdef	BIT_FIELD
#define T16ACH0_CCA						*(volatile unsigned short *)(BASE_ADDR+0x1406)
	#define T16ACH0_CCA0							T16ACH0_CCA						// T16A2 CH0 Attached counter select

#define T16ACH1_CCA						*(volatile unsigned short *)(BASE_ADDR+0x1426)
	#define T16ACH1_CCA1							T16ACH1_CCA						// T16A2 CH1 Attached counter select
#endif	// end of BIT_FIELD


// PWM Timer(T16A2) Comparator/Capture B Data Register
#define	R16_T16A_CCB(ch)					(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x08+(ch*T16A2_CHOFFSET))

#ifdef	BIT_FIELD
#define T16ACH0_CCB						*(volatile unsigned short *)(BASE_ADDR+0x1408)
	#define T16ACH0_CCB0							T16ACH0_CCB						// T16A2 CH0 Attached counter select

#define T16ACH1_CCB							*(volatile unsigned short *)(BASE_ADDR+0x1428)
	#define T16ACH1_CCB1							T16ACH1_CCB						// T16A2 CH1 Attached counter select
#endif	// end of BIT_FIELD


// PWM Timer(T16A2) Comparator/Capture Interrupt Enable Register
#define	R16_T16A_IEN(ch)					(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x0a+(ch*T16A2_CHOFFSET))
	#define	CAPBOWIE_SFT						5
	#define	CAPAOWIE_SFT						4
	#define	CAPBIE_SFT							3
	#define	CAPAIE_SFT							2
	#define	CBIE_SFT							1
	#define	CAIE_SFT							0
	#define	CAPBOWIE							BIT(0x0001, CAPBOWIE_SFT)		// Capture B Overwrite Interrupt Enable
	#define	CAPAOWIE							BIT(0x0001, CAPAOWIE_SFT)		// Capture A Overwrite Interrupt Enable
	#define	CAPBIE								BIT(0x0001, CAPBIE_SFT)			// Capture B Interrupt Enable
	#define	CAPAIE								BIT(0x0001, CAPAIE_SFT)			// Capture A Interrupt Enable
	#define	CBIE								BIT(0x0001, CBIE_SFT)			// Compare B Interrupt Enable
	#define	CAIE								BIT(0x0001, CAIE_SFT)			// Compare A Interrupt Enable
		#define	T16A_INT_MASK						(CAPBOWIE | CAPAOWIE | CAPBIE | CAPAIE | CBIE | CAIE)	// interrupt mask data

#ifdef	BIT_FIELD
#define T16ACH0_IEN							*(volatile unsigned short *)(BASE_ADDR+0x140A)
#define BF_T16ACH0_IEN						(*(volatile struct t16a_ienBitF *)(BASE_ADDR+0x140A))
	#define T16ACH0_IEN_CAPBOWIE0					BF_T16ACH0_IEN.capbowie			// T16A2 CH0 Capture B over write interrupt enable
	#define T16ACH0_IEN_CAPAOWIE0					BF_T16ACH0_IEN.capaowie			// T16A2 CH0 Capture A over write interrupt enable
	#define T16ACH0_IEN_CAPBIE0					BF_T16ACH0_IEN.capbie			// T16A2 CH0 Capture B interrupt enable
	#define T16ACH0_IEN_CAPAIE0					BF_T16ACH0_IEN.capaie			// T16A2 CH0 Capture A interrupt enable
	#define T16ACH0_IEN_CBIE0						BF_T16ACH0_IEN.cbie				// T16A2 CH0 Compare B interrupt enable
	#define T16ACH0_IEN_CAIE0						BF_T16ACH0_IEN.caie				// T16A2 CH0 Compare A interrupt enable

#define T16ACH1_IEN							*(volatile unsigned short *)(BASE_ADDR+0x142A)
#define BF_T16ACH1_IEN						(*(volatile struct t16a_ienBitF *)(BASE_ADDR+0x142A))
	#define T16ACH1_IEN_CAPBOWIE1					BF_T16ACH1_IEN.capbowie			// T16A2 CH1 Capture B over write interrupt enable
	#define T16ACH1_IEN_CAPAOWIE1					BF_T16ACH1_IEN.capaowie			// T16A2 CH1 Capture A over write interrupt enable
	#define T16ACH1_IEN_CAPBIE1					BF_T16ACH1_IEN.capbie			// T16A2 CH1 Capture B interrupt enable
	#define T16ACH1_IEN_CAPAIE1					BF_T16ACH1_IEN.capaie			// T16A2 CH1 Capture A interrupt enable
	#define T16ACH1_IEN_CBIE1						BF_T16ACH1_IEN.cbie				// T16A2 CH1 Compare B interrupt enable
	#define T16ACH1_IEN_CAIE1						BF_T16ACH1_IEN.caie				// T16A2 CH1 Compare A interrupt enable
#endif	// end of BIT_FIELD


// PWM Timer(T16A2) Comparator/Capture Interrupt Flag Register
#define	R16_T16A_IFLG(ch)					(volatile unsigned short *)(unsigned long)(T16A_BASE_ADDR+0x0c+(ch*T16A2_CHOFFSET))
	#define	CAPBOWIF_SFT						5
	#define	CAPAOWIF_SFT						4
	#define	CAPBIF_SFT							3
	#define	CAPAIF_SFT							2
	#define	CBIF_SFT							1
	#define	CAIF_SFT							0
	#define	CAPBOWIF							BIT(0x0001, CAPBOWIF_SFT)		// Capture B Overwrite Interrupt Flag
	#define	CAPAOWIF							BIT(0x0001, CAPAOWIF_SFT)		// Capture A Overwrite Interrupt Flag
	#define	CAPBIF								BIT(0x0001, CAPBIF_SFT)			// Capture B Interrupt Flag
	#define	CAPAIF								BIT(0x0001, CAPAIF_SFT)			// Capture A Interrupt Flag
	#define	CBIF								BIT(0x0001, CBIF_SFT)			// Compare B Interrupt Flag
	#define	CAIF								BIT(0x0001, CAIF_SFT)			// Compare A Interrupt Flag
		#define	T16A_NOTOCCURRED					0x0000							// not occurred
		#define	T16A_OCCURRED						0x0001							// occurred

#ifdef	BIT_FIELD
#define T16ACH0_IFLG						*(volatile unsigned short *)(BASE_ADDR+0x140C)
#define BF_T16ACH0_IFLG						(*(volatile struct t16a_iflgBitF *)(BASE_ADDR+0x140C))
	#define T16ACH0_IFLG_CAPBOWIF0					BF_T16ACH0_IFLG.capbowif		// T16A2 CH0 Capture B over write interrupt enable
	#define T16ACH0_IFLG_CAPAOWIF0					BF_T16ACH0_IFLG.capaowif		// T16A2 CH0 Capture A over write interrupt enable
	#define T16ACH0_IFLG_CAPBIF0					BF_T16ACH0_IFLG.capbif			// T16A2 CH0 Capture B interrupt enable
	#define T16ACH0_IFLG_CAPAIF0					BF_T16ACH0_IFLG.capaif			// T16A2 CH0 Capture A interrupt enable
	#define T16ACH0_IFLG_CBIF0						BF_T16ACH0_IFLG.cbif			// T16A2 CH0 Compare B interrupt enable
	#define T16ACH0_IFLG_CAIF0						BF_T16ACH0_IFLG.caif			// T16A2 CH0 Compare A interrupt enable

#define T16ACH1_IFLG						*(volatile unsigned short *)(BASE_ADDR+0x142C)
#define BF_T16ACH1_IFLG						(*(volatile struct t16a_iflgBitF *)(BASE_ADDR+0x142C))
	#define T16ACH1_IFLG_CAPBOWIF1					BF_T16ACH1_IFLG.capbowif		// T16A2 CH1 Capture B over write interrupt enable
	#define T16ACH1_IFLG_CAPAOWIF1					BF_T16ACH1_IFLG.capaowif		// T16A2 CH1 Capture A over write interrupt enable
	#define T16ACH1_IFLG_CAPBIF1					BF_T16ACH1_IFLG.capbif			// T16A2 CH1 Capture B interrupt enable
	#define T16ACH1_IFLG_CAPAIF1					BF_T16ACH1_IFLG.capaif			// T16A2 CH1 Capture A interrupt enable
	#define T16ACH1_IFLG_CBIF1						BF_T16ACH1_IFLG.cbif			// T16A2 CH1 Compare B interrupt enable
	#define T16ACH1_IFLG_CAIF1						BF_T16ACH1_IFLG.caif			// T16A2 CH1 Compare A interrupt enable
#endif	// end of BIT_FIELD

#endif /* T16A2_H_ */
