/******************************************************************
 * t8osc1.h
 * 
 * 8-bit OSC1 timer driver
 * 
 * Copyright(C) SEIKO EPSON CORPORATION 2009. All rights Reserved.
 *****************************************************************/

#ifndef T8O_H_
#define T8O_H_


#include <s1c176xx.h>


#ifdef	BIT_FIELD
/***********************************************
 ****** The Definition of bit field         ****
 ***********************************************/
struct t8o_ctlBitF {
	unsigned int	t8orun	: 1;
	unsigned int	t8ormd	: 1;
	unsigned int			: 2;
	unsigned int	t8otrst	: 1;
	unsigned int			: 3;
};

struct t8o_imskBitF {
	unsigned int	t8oie	: 1;
	unsigned int			: 7;
};

struct t8o_iflgBitF {
	unsigned int	t8oif	: 1;
	unsigned int			: 7;
};
#endif	// end of BIT_FIELD


/***********************************************
 ****** The Definition of base address      ****
 ***********************************************/
#define	T8O_BASE_ADDR						(BASE_ADDR+T8OSC1_OFFSET)


/***********************************************
 ****** The Definition of 8-bit OSC1 Timer *****
 ****** [T8OSC1]                           *****
 ***********************************************/
// 8-bit OSC1 Timer Control Register
#define	R08_T8OSC1_CTL						(volatile unsigned char *)(unsigned long)(T8O_BASE_ADDR+0x00)
	#define	T8ORST_SFT							4
	#define	T8ORMD_SFT							1
	#define	T8ORUN_SFT							0
	#define	T8ORST								BIT(0x01, T8ORST_SFT)		// Timer Reset
		#define	T8O_INVALID							0x00						// invalid
		#define	T8O_RESET							0x01						// reset
	#define	T8ORMD								BIT(0x01, T8ORMD_SFT)		// Count Mode Select
		#define	T8O_REPEAT							0x00						// repeat
		#define	T8O_ONESHOT							0x01						// one shot
	#define	T8ORUN								BIT(0x01, T8ORUN_SFT)		// Timer Run/Stop Control
		#define	T8O_STOP							0x00						// stop
		#define	T8O_RUN								0x01						// run

#ifdef	BIT_FIELD
#define T8OSC1_CTL							*(volatile unsigned char *)(BASE_ADDR+0x10c0)
#define BF_T8OSC1_CTL						(*(volatile struct t8o_ctlBitF *)(BASE_ADDR+0x10c0))
	#define T8OSC1_CTL_T8ORST					BF_T8OSC1_CTL.t8otrst		// Timer reset
	#define T8OSC1_CTL_T8ORMD					BF_T8OSC1_CTL.t8ormd		// Count mode select
	#define T8OSC1_CTL_T8ORUN					BF_T8OSC1_CTL.t8orun		// Timer run/stop control
#endif	// end of BIT_FIELD


// 8-bit OSC1 Timer Counter Data Register
#define	R08_T8OSC1_CNT						(volatile unsigned char *)(unsigned long)(T8O_BASE_ADDR+0x01)

#ifdef	BIT_FIELD
#define T8OSC1_CNT							*(volatile unsigned char *)(BASE_ADDR+0x10c1)
	#define T8OSC1_CNT_T8OCNT					T8OSC1_CNT					// Timer counter data
#endif	// end of BIT_FIELD


// 8-bit OSC1 Timer Compare Data Register
#define	R08_T8OSC1_CMP						(volatile unsigned char *)(unsigned long)(T8O_BASE_ADDR+0x02)

#ifdef	BIT_FIELD
#define T8OSC1_CMP							*(volatile unsigned char *)(BASE_ADDR+0x10c2)
	#define T8OSC1_CMP_T8OCMP					T8OSC1_CMP					// Compare data
#endif	// end of BIT_FIELD

// 8-bit OSC1 Timer Interrupt Mask Register
#define	R08_T8OSC1_IMSK						(volatile unsigned char *)(unsigned long)(T8O_BASE_ADDR+0x03)
	#define	T8OIE_SFT							0
	#define	T8OIE								BIT(0x01, T8OIE_SFT)		// 8-bit OSC1 Timer Interrupt Enable
		#define	T8O_DISABLE							0x00						// disable
		#define	T8O_ENABLE							0x01						// enable

#ifdef	BIT_FIELD
#define T8OSC1_IMSK							*(volatile unsigned char *)(BASE_ADDR+0x10c3)
#define BF_T8OSC1_IMSK						(*(volatile struct t8o_imskBitF *)(BASE_ADDR+0x10c3))
	#define T8OSC1_IMSK_T8OIE					BF_T8OSC1_IMSK.t8oie		// 8-bit OSC1 timer interrupt enable
#endif	// end of BIT_FIELD


// 8-bit OSC1 Timer Interrupt Flag Register
#define	R08_T8OSC1_IFLG						(volatile unsigned char *)(unsigned long)(T8O_BASE_ADDR+0x04)
	#define	T8OIF_SFT							0
	#define	T8OIF								BIT(0x01, T8OIF_SFT)		// 8-bit OSC1 Timer Interrupt Flag
		#define	T8O_NOTOCCURRED						0x00						// not occurred
		#define	T8O_OCCURRED						0x01						// occurred

#ifdef	BIT_FIELD
#define T8OSC1_IFLG							*(volatile unsigned char *)(BASE_ADDR+0x10c4)
#define BF_T8OSC1_IFLG						(*(volatile struct t8o_iflgBitF *)(BASE_ADDR+0x10c4))
	#define T8OSC1_IFLG_T8OIF					BF_T8OSC1_IFLG.t8oif		// 8-bit OSC1 timer interrupt flag  (read only)
#endif	// end of BIT_FIELD


// 8-bit OSC1 Timer PWM Duty Data Register
#define	R08_T8OSC1_DUTY						(volatile unsigned char *)(unsigned long)(T8O_BASE_ADDR+0x05)

#ifdef	BIT_FIELD
#define T8OSC1_DUTY							*(volatile unsigned char *)(BASE_ADDR+0x10c5)
	#define T8OSC1_DUTY_T8ODTY					T8OSC1_DUTY					// 8-bit OSC1 timer PWM Duty data
#endif	// end of BIT_FILED


#endif	// end of T8O_H_
